import { NgModule } from "@angular/core";
import { Routes, RouterModule } from "@angular/router";
import { LoginComponent } from "./components/login/login.component";
import { RegistroComponent } from "./components/registro/registro.component";
import { HomeComponent } from "./components/home/home.component";
import { CanActivateUsuarioLoggedService } from "./utils/can-activate-usuario-logged.service";
import { TareasComponent } from "./components/tareas/tareas.component";
import { ParcelasComponent } from "./components/parcelas/parcelas.component";
import { AperosComponent } from "./components/aperos/aperos.component";
import { VerUsuariosAgendaComponent } from "./components/ver-usuarios-agenda/ver-usuarios-agenda.component";
import { CanActivateAgricultorGuard } from "./utils/can-activate-agricultor.guard";
import { CanActivateCooperativaGuard } from "./utils/can-activate-cooperativa.guard";
import { CanActivateAdminGuard } from "./utils/can-activate-admin.guard";
import { SociosCooperativaComponent } from "./components/socios-cooperativa/socios-cooperativa.component";
import { CooperativaComponent } from "./components/cooperativa/cooperativa.component";
import { ProductosCooperativaComponent } from "./components/productos-cooperativa/productos-cooperativa.component";
import { UsuariosComponent } from "./components/usuarios/usuarios.component";
import { RecuperarPasswordComponent } from "./components/recuperar-password/recuperar-password.component";

const routes: Routes = [
  { path: "login", component: LoginComponent },
  { path: "registro", component: RegistroComponent },
  { path: "recuperarPassword", component: RecuperarPasswordComponent },
  {
    path: "agricultor",
    component: HomeComponent,
    canActivate: [CanActivateUsuarioLoggedService],
    canActivateChild: [CanActivateAgricultorGuard],
    children: [
      { path: "", redirectTo: "tareas", pathMatch: "full" },
      { path: "tareas", component: TareasComponent },
      { path: "parcelas", component: ParcelasComponent },
      { path: "aperos", component: AperosComponent },
      { path: "agenda", component: VerUsuariosAgendaComponent },
      { path: "cooperativa", component: CooperativaComponent }
    ]
  },
  {
    path: "cooperativa",
    component: HomeComponent,
    canActivate: [CanActivateUsuarioLoggedService],
    canActivateChild: [CanActivateCooperativaGuard],
    children: [
      { path: "", redirectTo: "productos", pathMatch: "full" },
      { path: "socios", component: SociosCooperativaComponent },
      { path: "productos", component: ProductosCooperativaComponent }
    ]
  },
  {
    path: "admin",
    component: HomeComponent,
    canActivate: [CanActivateUsuarioLoggedService],
    canActivateChild: [CanActivateAdminGuard],
    children: [
      { path: "", redirectTo: "usuarios", pathMatch: "full" },
      { path: "usuarios", component: UsuariosComponent }
    ]
  },

  { path: "**", redirectTo: "login" }
];

@NgModule({
  imports: [RouterModule.forRoot(routes)],
  exports: [RouterModule]
})
export class AppRoutingModule {}
