import { HttpErrorResponse } from "@angular/common/http";
import { Component, Inject, OnDestroy, OnInit } from "@angular/core";
import { FormControl, FormGroup, Validators } from "@angular/forms";
import { MatDialogRef, MAT_DIALOG_DATA } from "@angular/material/dialog";
import { Subscription } from "rxjs";
import { Constantes } from "src/app/constantes";
import { Usuario } from "src/app/models/usuario";
import { PrivateAgricultorService } from "src/app/services/privateagricultor.service";
import { SnackbarService } from "src/app/services/snackbaralerts.service";
import { opcionSeleccionadaValidator } from "src/app/validators/validators";

@Component({
  selector: "app-add-usuario",
  templateUrl: "./add-usuario.component.html",
  styleUrls: ["./add-usuario.component.css"]
})
export class AddUsuarioComponent implements OnInit, OnDestroy {
  public formAddUsuario: FormGroup;
  public agricultores: Usuario[];
  public agricultoresCopia: Usuario[] = [];
  private subscription: Subscription = new Subscription();
  constructor(
    private privateAgricultorservice: PrivateAgricultorService,
    private snackbarService: SnackbarService,
    private matDialogRef: MatDialogRef<AddUsuarioComponent>,
    @Inject(MAT_DIALOG_DATA) public data
  ) {}

  ngOnInit() {
    const getAgricultores$ = this.privateAgricultorservice.getAgricultores();
    this.formAddUsuario = new FormGroup({
      nombre: new FormControl(
        "",
        Validators.required,
        opcionSeleccionadaValidator()
      )
    });
    this.subscription.add(
      getAgricultores$.subscribe(
        data => {
          const userLogged: Usuario = JSON.parse(
            sessionStorage.getItem(Constantes.USER_LOCAL_STORAGE)
          );
          this.agricultores = data;
          let indice = -1;
          this.agricultores.forEach((usuario, index) => {
            if (usuario.identificador === userLogged.identificador) {
              indice = index;
            }
          });
          if (indice !== -1) {
            this.agricultores.splice(indice, 1);
          }
        },
        error => {
          this.snackbarService.mostrar(
            "No se ha podido cargar los datos.",
            "Ok",
            "snackbar-alert"
          );
          this.matDialogRef.close(this.data);
        }
      )
    );
  }

  public filtrar(value: string) {
    if (value.length > 2) {
      this.agricultoresCopia = this.agricultores.filter(
        agricultor =>
          agricultor.nombre.toLowerCase().includes(value.toLowerCase()) ||
          agricultor.username.toLowerCase().includes(value.toLowerCase())
      );
    }
  }

  public getNombre(item?: Usuario) {
    return item ? `${item.nombre} (${item.username})` : "";
  }

  public addUsuario() {
    const usuario = this.formAddUsuario.controls["nombre"].value;
    const addUsuario$ = this.privateAgricultorservice.addUsuarioAgenda(usuario);
    this.subscription.add(
      addUsuario$.subscribe(
        (data: string) => {
          this.snackbarService.mostrar("Usuario añadido", "Ok");
          this.matDialogRef.close({ added: true, usuario: usuario });
        },
        (error: HttpErrorResponse) => {
          if (error.status === 0) {
            this.snackbarService.mostrar(
              "Error a conectar con el servidor.",
              "Ok",
              "snackbar-alert"
            );
          }
          if (error.status === 400) {
            this.snackbarService.mostrar(error.error, "Ok", "snackbar-alert");
          }
          this.matDialogRef.close(this.data);
        }
      )
    );
  }

  public cerrarForm() {
    this.snackbarService.mostrar(
      "No se ha añadido ningún usuario.",
      "Ok",
      "snackbar-alert"
    );
    this.matDialogRef.close(this.data);
  }

  ngOnDestroy() {
    this.subscription.unsubscribe();
  }
}
