import { Component, OnDestroy, OnInit, ViewChild } from "@angular/core";
import { MatDialog } from "@angular/material/dialog";
import { MatSort } from "@angular/material/sort";
import { MatTableDataSource } from "@angular/material/table";
import { Subscription } from "rxjs";
import { Constantes } from "src/app/constantes";
import { Apero } from "src/app/models/apero";
import { UsuarioAgricultor } from "src/app/models/usuario-agricultor";
import { PrivateAgricultorService } from "src/app/services/privateagricultor.service";
import { SnackbarService } from "src/app/services/snackbaralerts.service";
import { NuevoAperoComponent } from "../nuevoapero/nuevoapero.component";
import { VerDetallesAperoComponent } from "../ver-detalles-apero/ver-detalles-apero.component";

@Component({
  selector: "app-aperos",
  templateUrl: "./aperos.component.html",
  styleUrls: ["./aperos.component.css"]
})
export class AperosComponent implements OnInit, OnDestroy {
  constructor(
    private privateAgricultorService: PrivateAgricultorService,
    private snackbarService: SnackbarService,
    private matDialog: MatDialog
  ) {}

  @ViewChild(MatSort, { static: true }) sort: MatSort;

  private subscriptions: Subscription = new Subscription();
  public columnas: string[] = [
    "marca",
    "modelo",
    "matricula",
    "tipoLabores",
    "detalles"
  ];
  public aperos = [];
  public dataSource: MatTableDataSource<Apero>;

  ngOnInit() {
    const usuarioLogged: UsuarioAgricultor = JSON.parse(
      sessionStorage.getItem(Constantes.USER_LOCAL_STORAGE)
    );
    const aperos$ = this.privateAgricultorService.getAperosAgricultor(
      usuarioLogged.identificador
    );
    this.subscriptions.add(
      aperos$.subscribe(
        (data: Apero[]) => {
          this.aperos = data;
          this.dataSource = new MatTableDataSource(this.aperos);
          this.dataSource.sort = this.sort;
        },
        error => {
          this.aperos = [];
          this.dataSource = new MatTableDataSource(this.aperos);
          this.snackbarService.mostrar(
            "No se ha podido cargar los aperos.",
            "Ok",
            "snackbar-alert"
          );
        }
      )
    );
  }

  ngOnDestroy(): void {
    this.subscriptions.unsubscribe();
  }

  public abrirNuevoApero() {
    const data = { creado: false, apero: {} };
    let dialog = this.matDialog.open(NuevoAperoComponent, {
      data: data,
      height: "95%",
      width: "65%"
    });
    const cerrardialog$ = dialog.afterClosed();
    this.subscriptions.add(
      cerrardialog$.subscribe(
        data => {
          if (data.creado) {
            this.aperos.push(data.apero);
            this.dataSource.data = this.aperos;
            this.snackbarService.mostrar("Apero registrado.", "Ok");
          } else {
            this.snackbarService.mostrar(
              "No se ha podido registrar el apero.",
              "Ok",
              "snackbar-alert"
            );
          }
        },
        error => {
          this.snackbarService.mostrar(
            "No se ha podido registrar el apero.",
            "Ok",
            "snackbar-alert"
          );
        }
      )
    );
  }

  public detallesApero(apero: Apero) {
    const dialog = this.matDialog.open(VerDetallesAperoComponent, {
      data: apero,
      height: "95%",
      width: "60%"
    });
  }
}
