import { Component, OnInit, OnDestroy } from "@angular/core";
import { FormGroup, FormControl, Validators } from "@angular/forms";
import { Constantes } from "../../constantes";
import { PublicService } from "src/app/services/public.service";
import { HttpErrorResponse } from "@angular/common/http";
import { JwtHelperService } from "@auth0/angular-jwt";
import { ITokenResponse } from "src/app/models/itoken-response";
import { Usuario, Role } from "src/app/models/usuario";
import { Router } from "@angular/router";
import { Observable, Subscription } from "rxjs";

@Component({
  selector: "app-login",
  templateUrl: "./login.component.html",
  styleUrls: ["./login.component.css"]
})
export class LoginComponent implements OnInit, OnDestroy {
  loginForm: FormGroup;
  mensaje: string;

  nombreAPP = Constantes.NOMBRE_APP;

  subscriptions: Subscription;
  login$: Observable<any>;

  constructor(private publicService: PublicService, private router: Router) {}

  ngOnInit() {
    sessionStorage.clear();
    this.subscriptions = new Subscription();
    this.loginForm = new FormGroup({
      username: new FormControl("", [Validators.required, Validators.email]),
      password: new FormControl("", Validators.required)
    });
  }

  ngOnDestroy(): void {
    this.subscriptions.unsubscribe();
  }

  public login() {
    this.mensaje = "";
    const username: string = this.loginForm.controls["username"].value;
    const password: string = this.loginForm.controls["password"].value;

    this.login$ = this.publicService.login(username, password);
    this.subscriptions.add(
      this.login$.subscribe(
        (data: ITokenResponse) => {
          sessionStorage.setItem(Constantes.TOKEN_LOCAL_STORAGE, data.token);
          const tokenDecoded = new JwtHelperService().decodeToken(data.token);
          const iUsuarioLogged: Usuario = JSON.parse(tokenDecoded.sub);
          sessionStorage.setItem(
            Constantes.USER_LOCAL_STORAGE,
            JSON.stringify(iUsuarioLogged)
          );
          switch (iUsuarioLogged.rol) {
            case Role.ROLE_AGRICULTOR:
              this.router.navigate(["/agricultor/"]);
              break;
            case Role.ROLE_RESPONSABLE_COOPERATIVA:
              this.router.navigate(["/cooperativa/"]);
              break;
            case Role.ROLE_ADMIN:
              this.router.navigate(["/admin/"]);
              break;
          }
        },
        error => {
          if (error instanceof HttpErrorResponse) {
            if (error.status === 403) {
              if (error.error.error.includes("locked")) {
                this.mensaje =
                  "La cuenta ha sido bloqueada. Póngase en contacto con el administrador.";
              } else {
                this.mensaje = "Usuario o contraseña incorrectos.";
              }
            } else {
              if (error.status === 0) {
                this.mensaje = "Error al conectar con el servidor.";
              }
            }
          }
        }
      )
    );
  }
}
