import { Component, Inject, OnDestroy, OnInit } from "@angular/core";
import { FormControl, FormGroup, Validators } from "@angular/forms";
import { MatDialogRef, MAT_DIALOG_DATA } from "@angular/material/dialog";
import { Subscription } from "rxjs";
import { Constantes } from "src/app/constantes";
import { Parcela } from "src/app/models/parcela";
import { UsuarioAgricultor } from "src/app/models/usuario-agricultor";
import { PrivateAgricultorService } from "src/app/services/privateagricultor.service";

@Component({
  selector: "app-nueva-parcela",
  templateUrl: "./nueva-parcela.component.html",
  styleUrls: ["./nueva-parcela.component.css"]
})
export class NuevaParcelaComponent implements OnInit, OnDestroy {
  formCrearParcela: FormGroup;
  private subcriptions: Subscription = new Subscription();

  constructor(
    private privateAgricultorService: PrivateAgricultorService,
    @Inject(MAT_DIALOG_DATA) private data,
    private dialogRef: MatDialogRef<NuevaParcelaComponent>
  ) {}

  ngOnInit() {
    this.formCrearParcela = new FormGroup({
      nombre: new FormControl("", Validators.required),
      descripcion: new FormControl(""),
      localidad: new FormControl("", Validators.required),
      superficieHa: new FormControl("", [
        Validators.required,
        Validators.min(0)
      ]),
      coordLat: new FormControl(""),
      coordLon: new FormControl("")
    });
  }

  ngOnDestroy() {
    this.subcriptions.unsubscribe();
  }

  public actualizarPosicion(lat, long) {
    this.formCrearParcela.controls["coordLon"].setValue(long);
    this.formCrearParcela.controls["coordLat"].setValue(lat);
  }

  public crearParcela() {
    const nombre = this.formCrearParcela.controls["nombre"].value;
    const descripcion = this.formCrearParcela.controls["descripcion"].value;
    const localidad = this.formCrearParcela.controls["localidad"].value;
    const superficie = this.formCrearParcela.controls["superficieHa"].value;
    const coordLat = this.formCrearParcela.controls["coordLat"].value;
    const coordLong = this.formCrearParcela.controls["coordLon"].value;
    const usuarioLogged: UsuarioAgricultor = JSON.parse(
      sessionStorage.getItem(Constantes.USER_LOCAL_STORAGE)
    );
    const parcela: Parcela = {
      nombre: nombre,
      descripcion: descripcion,
      localidad: localidad,
      superficieHa: superficie,
      coordLat: coordLat,
      coordLong: coordLong,
      propietario: usuarioLogged
    };
    const registrarParcela$ = this.privateAgricultorService.registrarParcela(
      parcela
    );
    this.subcriptions.add(
      registrarParcela$.subscribe(
        data => {
          this.data.creado = true;
          this.data.parcela = data;
          this.dialogRef.close(this.data);
        },
        error => {
          this.dialogRef.close(this.data);
        }
      )
    );
  }
}
