import { Component, OnInit, Inject, OnDestroy } from '@angular/core';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { PrivateCooperativaService } from 'src/app/services/private-cooperativa.service';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { Subscription } from 'rxjs';
import { ProductoCooperativa } from 'src/app/models/producto-cooperativa';

@Component({
  selector: 'app-nuevo-producto',
  templateUrl: './nuevo-producto.component.html',
  styleUrls: ['./nuevo-producto.component.css']
})
export class NuevoProductoComponent implements OnInit, OnDestroy {

  public formNuevoProducto: FormGroup;

  private subscriptions: Subscription = new Subscription();

  constructor(@Inject(MAT_DIALOG_DATA) private data, private matDialogRef: MatDialogRef<NuevoProductoComponent>, private privateCooperativaService: PrivateCooperativaService) { }

  ngOnInit() {
    this.formNuevoProducto = new FormGroup({
      'nombreProducto': new FormControl('', Validators.required),
      'precio': new FormControl('', [Validators.required, Validators.min(0)])
    });
  }

  public crearProducto() {
    const nombre = this.formNuevoProducto.controls['nombreProducto'].value;
    const precio = this.formNuevoProducto.controls['precio'].value;
    const fecha = new Date();
    const registrarProducto$ = this.privateCooperativaService.registrarNuevoProducto({ nombre: nombre, fechaModificacion: fecha, preciosProductos: [{ precio: precio, fechaCreacion: fecha }] });
    this.subscriptions.add(
      registrarProducto$.subscribe(
        (data: ProductoCooperativa) => {
          this.data.creado = true;
          this.data.producto = data;
          this.matDialogRef.close(this.data)
        },
        (error) => {
          this.matDialogRef.close(this.data);
        }
      )
    )
  }

  ngOnDestroy() {
    this.subscriptions.unsubscribe();
  }

}
