import { Component, Inject, OnDestroy, OnInit } from "@angular/core";
import { FormArray, FormControl, FormGroup, Validators } from "@angular/forms";
import { MatCheckboxChange } from "@angular/material/checkbox";
import { MAT_DIALOG_DATA, MatDialogRef } from "@angular/material/dialog";
import { Subscription } from "rxjs";
import { Constantes } from "src/app/constantes";
import { Apero } from "src/app/models/apero";
import { TipoLabor } from "src/app/models/tipolabor";
import { PrivateAgricultorService } from "src/app/services/privateagricultor.service";
import { PrivatetiposlaborService } from "src/app/services/privatetiposlabor.service";
import { SnackbarService } from "src/app/services/snackbaralerts.service";

@Component({
  selector: "app-nuevoapero",
  templateUrl: "./nuevoapero.component.html",
  styleUrls: ["./nuevoapero.component.css"]
})
export class NuevoAperoComponent implements OnInit, OnDestroy {
  public formNuevoApero: FormGroup;
  public tiposLabores: TipoLabor[];
  public subscriptions: Subscription = new Subscription();
  private laboresSeleccionadas: boolean[] = [];

  constructor(
    private snackbarService: SnackbarService,
    private tiposLaboresService: PrivatetiposlaborService,
    private privateAgricultorService: PrivateAgricultorService,
    @Inject(MAT_DIALOG_DATA) public data,
    public dialogRef: MatDialogRef<NuevoAperoComponent>
  ) {}

  ngOnInit() {
    this.formNuevoApero = new FormGroup({
      marca: new FormControl("", Validators.required),
      modelo: new FormControl("", Validators.required),
      descripcion: new FormControl(""),
      matricula: new FormControl(
        "",
        Validators.pattern("^E [0-9]{4} [A-Z]{3}$")
      ),
      precioHa: new FormControl("", [Validators.required, Validators.min(0)]),
      tiposLabores: new FormArray([])
    });
    const tiposLabores$ = this.tiposLaboresService.getTiposLabor();
    this.subscriptions.add(
      tiposLabores$.subscribe(
        data => {
          this.tiposLabores = data;
          this.buildFormArrayTipoLabores();
        },
        error => {
          this.snackbarService.mostrar(
            "No se ha podido cargar los tipos de labores",
            "OK",
            "snackbar-alert"
          );
          this.tiposLabores = [];
        }
      )
    );
  }

  public get formArrayTipoLabores(): FormArray {
    return this.formNuevoApero.controls["tiposLabores"] as FormArray;
  }

  public buildFormArrayTipoLabores() {
    const control = this.formArrayTipoLabores;
    this.tiposLabores.forEach(item => {
      control.push(new FormControl());
      this.laboresSeleccionadas.push(false);
    });
    return control;
  }

  public actualizarTiposLabores(i: MatCheckboxChange) {
    this.laboresSeleccionadas[i.source.value] = i.checked;
  }

  public registrarApero() {
    const usuarioLogged = JSON.parse(
      sessionStorage.getItem(Constantes.USER_LOCAL_STORAGE)
    );
    const marca = this.formNuevoApero.controls["marca"].value;
    const modelo = this.formNuevoApero.controls["modelo"].value;
    const descripcion = this.formNuevoApero.controls["descripcion"].value;
    const matricula = this.formNuevoApero.controls["matricula"].value;
    const precioHa = this.formNuevoApero.controls["precioHa"].value;
    let tiposLabores = [];
    this.laboresSeleccionadas.forEach((item, index) => {
      if (item) {
        tiposLabores.push(this.tiposLabores[index]);
      }
    });
    const apero: Apero = {
      marca: marca,
      modelo: modelo,
      descripcion: descripcion,
      matricula: matricula,
      costePorHa: precioHa,
      propietario: usuarioLogged,
      tipoLabores: tiposLabores
    };
    const registrarApero$ = this.privateAgricultorService.registrarApero(apero);
    this.subscriptions.add(
      registrarApero$.subscribe(
        data => {
          this.data.creado = true;
          this.data.apero = data;
          this.dialogRef.close(this.data);
        },
        error => {
          this.data.creado = false;
          this.dialogRef.close(this.data);
        }
      )
    );
  }

  ngOnDestroy() {
    this.subscriptions.unsubscribe();
  }
}
