import { Component, OnDestroy, OnInit, ViewChild } from "@angular/core";
import { MatDialog } from "@angular/material/dialog";
import { MatSort } from "@angular/material/sort";
import { MatTableDataSource } from "@angular/material/table";
import { Subscription } from "rxjs";
import { Constantes } from "src/app/constantes";
import { Parcela } from "src/app/models/parcela";
import { UsuarioAgricultor } from "src/app/models/usuario-agricultor";
import { PrivateAgricultorService } from "src/app/services/privateagricultor.service";
import { SnackbarService } from "src/app/services/snackbaralerts.service";
import { tieneUbicacion } from "src/app/pipes/tiene-ubicacion.pipe";
import { NuevaParcelaComponent } from "../nueva-parcela/nueva-parcela.component";
import { VerDetallesParcelaComponent } from "../ver-detalles-parcela/ver-detalles-parcela.component";

@Component({
  selector: "app-parcelas",
  templateUrl: "./parcelas.component.html",
  styleUrls: ["./parcelas.component.css"]
})
export class ParcelasComponent implements OnInit, OnDestroy {
  private subscriptions: Subscription = new Subscription();
  public columnas: string[] = [
    "nombre",
    "localidad",
    "superficieHa",
    "detalles"
  ];
  @ViewChild(MatSort, { static: true }) sort: MatSort;
  public parcelas = [];
  public dataSource: MatTableDataSource<Parcela>;

  constructor(
    private privateAgricultorService: PrivateAgricultorService,
    private snackbarService: SnackbarService,
    private matDialog: MatDialog
  ) {}

  ngOnInit() {
    const usuarioLogged: UsuarioAgricultor = JSON.parse(
      sessionStorage.getItem(Constantes.USER_LOCAL_STORAGE)
    );
    const parcelas$ = this.privateAgricultorService.getParcelasAgricultor();
    this.subscriptions.add(
      parcelas$.subscribe(
        data => {
          this.parcelas = data;
          this.dataSource = new MatTableDataSource(data);
          this.dataSource.sort = this.sort;
        },
        error => {
          this.dataSource = new MatTableDataSource([]);
          this.snackbarService.mostrar(
            "No se ha podido cargar las parcelas.",
            "Ok",
            "snackbar-alert"
          );
        }
      )
    );
  }

  public abrirNuevaParcela() {
    const data = { creado: false, parcela: {} };
    const dialog = this.matDialog.open(NuevaParcelaComponent, {
      data: data,
      height: "95%",
      width: "60%"
    });
    dialog.afterClosed().subscribe(
      data => {
        if (data.creado) {
          this.parcelas.push(data.parcela);
          this.dataSource.data = this.parcelas;
          this.snackbarService.mostrar("Parcela registrada.", "Ok");
        } else {
          this.snackbarService.mostrar(
            "No se ha podido registrar la parcela.",
            "Ok",
            "snackbar-alert"
          );
        }
      },
      error => {
        this.snackbarService.mostrar(
          "No se ha podido registrar la parcela.",
          "Ok",
          "snackbar-alert"
        );
      }
    );
  }

  public abrirDetallesParcela(identificador: number) {
    this.matDialog.open(VerDetallesParcelaComponent, {
      data: identificador,
      height: "95%",
      width: "60%"
    });
  }

  public getPrimeraParcelaUbicacion() {
    const parcelasConUbicacio: Parcela[] = this.parcelas.filter(item =>
      tieneUbicacion(item)
    );
    if (parcelasConUbicacio.length > 0) {
      return {
        lat: parcelasConUbicacio[0].coordLat,
        lon: parcelasConUbicacio[0].coordLong
      };
    } else {
      return { lat: 41.86461, lon: -5.41111 };
    }
  }

  ngOnDestroy() {
    this.subscriptions.unsubscribe();
  }
}
