import { Component, OnInit, OnDestroy } from "@angular/core";
import { FormGroup, FormControl, Validators } from "@angular/forms";
import { PublicService } from "src/app/services/public.service";
import { Subscription } from "rxjs";
import { repetirPasswordCorrectoValidator } from "src/app/validators/validators";
import { MatSnackBar } from "@angular/material/snack-bar";
import { Router } from "@angular/router";

@Component({
  selector: "app-recuperar-password",
  templateUrl: "./recuperar-password.component.html",
  styleUrls: ["./recuperar-password.component.css"]
})
export class RecuperarPasswordComponent implements OnInit, OnDestroy {
  formUsername: FormGroup;
  formNuevaPassword: FormGroup;
  mensaje = "";
  segundaParte = false;

  private subscription: Subscription = new Subscription();
  private usuario;

  constructor(
    private publicController: PublicService,
    private snackBar: MatSnackBar,
    private router: Router
  ) {}

  ngOnInit() {
    this.formUsername = new FormGroup({
      username: new FormControl("", [Validators.required, Validators.email])
    });
    this.formNuevaPassword = new FormGroup(
      {
        password: new FormControl("", [Validators.required]),
        passwordRepetida: new FormControl("", Validators.required)
      },
      [],
      repetirPasswordCorrectoValidator()
    );
  }

  public comprobarUsuario() {
    const username = this.formUsername.controls["username"].value;
    const comprobarUsername$ = this.publicController.comprobarUsername(
      username
    );
    this.subscription.add(
      comprobarUsername$.subscribe(data => {
        if (data) {
          this.mensaje = "";
          this.usuario = data;
          this.segundaParte = true;
        } else {
          this.mensaje =
            "No se ha encontrado ningún usuario con ese correo electrónico.";
        }
      })
    );
  }

  public cambiarPassword() {
    const passwordNueva = this.formNuevaPassword.controls["password"].value;
    this.usuario.password = passwordNueva;
    const modificarUsuario$ = this.publicController.modificarUsuario(
      this.usuario
    );
    this.subscription.add(
      modificarUsuario$.subscribe(
        data => {
          let snackbar = this.snackBar.open("Contraseña actualizada", "Ok");
          snackbar.afterDismissed().subscribe(
            data => {
              this.router.navigate(["/login"]);
            },
            error => {
              this.router.navigate(["/login"]);
            }
          );
        },
        error => {
          let snackbar = this.snackBar.open(
            "No se pudo actualizar la contraseña",
            "Ok",
            {
              panelClass: ["snackbar-alert"]
            }
          );
          snackbar.afterDismissed().subscribe(
            data => {
              this.router.navigate(["/login"]);
            },
            error => {
              this.router.navigate(["/login"]);
            }
          );
        }
      )
    );
  }

  public ngOnDestroy() {
    this.subscription.unsubscribe();
  }
}
