import { Component, OnDestroy, OnInit, ViewChild } from "@angular/core";
import { MatTable, MatTableDataSource } from "@angular/material/table";
import { Subscription } from "rxjs";
import { Usuario } from "src/app/models/usuario";
import { PrivateAdministradorService } from "src/app/services/private-administrador.service";
import { SnackbarService } from "src/app/services/snackbaralerts.service";

@Component({
  selector: "app-usuarios",
  templateUrl: "./usuarios.component.html",
  styleUrls: ["./usuarios.component.css"]
})
export class UsuariosComponent implements OnInit, OnDestroy {
  public dataSource: MatTableDataSource<Usuario>;
  private subscription: Subscription = new Subscription();
  public columnas = [
    "nombre",
    "username",
    "tipoUsuario",
    "estado",
    "modificar"
  ];
  @ViewChild(MatTable, { static: true }) table: MatTable<Usuario>;

  constructor(
    private privateAdminService: PrivateAdministradorService,
    private snackbarService: SnackbarService
  ) {}

  ngOnInit() {
    const getUsuarios$ = this.privateAdminService.getUsuarios();
    this.subscription.add(
      getUsuarios$.subscribe(
        data => {
          this.dataSource = new MatTableDataSource(data);
        },
        error => {
          this.dataSource = new MatTableDataSource([]);
          this.snackbarService.mostrar(
            "No se ha podido recuperar los datos.",
            "Ok",
            "snackbar-alert"
          );
        }
      )
    );
  }

  public bloquear(usuario: Usuario) {
    usuario.isBloqueado = true;
    const modificarUsuario$ = this.privateAdminService.modificarUsuario(
      usuario
    );
    this.subscription.add(
      modificarUsuario$.subscribe(
        data => {
          const index = this.dataSource.data.indexOf(usuario);
          if (index != -1) {
            this.dataSource.data[index] = usuario;
          }
          this.table.renderRows();
        },
        error => {
          usuario.isBloqueado = false;
          this.snackbarService.mostrar(
            "No se ha podido modificar el usuario.",
            "Ok",
            "snackbar-alert"
          );
        }
      )
    );
  }

  public desbloquear(usuario: Usuario) {
    usuario.isBloqueado = false;
    const modificarUsuario$ = this.privateAdminService.modificarUsuario(
      usuario
    );
    this.subscription.add(
      modificarUsuario$.subscribe(
        data => {
          this.table.renderRows();
          const index = this.dataSource.data.indexOf(usuario);
          if (index != -1) {
            this.dataSource.data[index] = usuario;
          }
        },
        error => {
          usuario.isBloqueado = true;
          this.snackbarService.mostrar(
            "No se ha podido modificar el usuario.",
            "Ok",
            "snackbar-alert"
          );
        }
      )
    );
  }

  public filtrar(value: string) {
    this.dataSource.filter = value.toLowerCase();
  }

  ngOnDestroy() {
    this.subscription.unsubscribe();
  }
}
