import { Component, OnInit, Inject, OnDestroy } from "@angular/core";
import { MAT_DIALOG_DATA } from "@angular/material/dialog";
import { Subscription } from "rxjs";
import { PrivateAgricultorService } from "src/app/services/privateagricultor.service";
import { Parcela } from "src/app/models/parcela";
import { SnackbarService } from "src/app/services/snackbaralerts.service";
import { switchMap, tap } from "rxjs/operators";
import { Tarea } from "src/app/models/tarea";
import { tieneUbicacion } from "src/app/pipes/tiene-ubicacion.pipe";

@Component({
  selector: "app-ver-detalles-parcela",
  templateUrl: "./ver-detalles-parcela.component.html",
  styleUrls: ["./ver-detalles-parcela.component.css"]
})
export class VerDetallesParcelaComponent implements OnInit, OnDestroy {
  constructor(
    @Inject(MAT_DIALOG_DATA) public identificadorParcela,
    private privateAgricultorService: PrivateAgricultorService,
    private snackbarService: SnackbarService
  ) {}

  private subscriptions: Subscription = new Subscription();
  public parcela: Parcela = null;
  public tareas: Tarea[] = [];

  ngOnInit() {
    const getParcela$ = this.privateAgricultorService
      .getParcela(this.identificadorParcela)
      .pipe(
        tap(data => (this.parcela = data)),
        switchMap(data =>
          this.privateAgricultorService.getTareasParcela(data.identificador)
        )
      );
    this.subscriptions.add(
      getParcela$.subscribe(
        (data: Tarea[]) => {
          this.tareas = data;
        },
        error => {
          this.snackbarService.mostrar(
            "No se ha podido cargar los datos de la parcela.",
            "Ok",
            "snackbar-alert"
          );
        }
      )
    );
  }

  ngOnDestroy() {
    this.subscriptions.unsubscribe();
  }

  private ordenarInverso(a: Tarea, b: Tarea) {
    return a.identificador > b.identificador ? -1 : 1;
  }

  public tieneCoordenadas() {
    return tieneUbicacion(this.parcela);
  }
}
