import { Component, OnInit, Inject, OnDestroy } from "@angular/core";
import { MAT_DIALOG_DATA, MatDialogRef } from "@angular/material/dialog";
import { ProductoCooperativa } from "src/app/models/producto-cooperativa";
import { FormGroup, FormControl, Validators } from "@angular/forms";
import { PrecioProducto } from "src/app/models/precio-producto";
import { Subscription } from "rxjs";
import { PrivateCooperativaService } from "src/app/services/private-cooperativa.service";
import { SnackbarService } from "src/app/services/snackbaralerts.service";

@Component({
  selector: "app-ver-detalles-producto",
  templateUrl: "./ver-detalles-producto.component.html",
  styleUrls: ["./ver-detalles-producto.component.css"]
})
export class VerDetallesProductoComponent implements OnInit, OnDestroy {
  public hoy = new Date();
  public producto: ProductoCooperativa;
  public formNuevoPrecio: FormGroup;

  private subscriptions: Subscription = new Subscription();

  constructor(
    @Inject(MAT_DIALOG_DATA) private data: ProductoCooperativa,
    private matDialogRef: MatDialogRef<VerDetallesProductoComponent>,
    private privateCoooperativa: PrivateCooperativaService,
    private snackbarService: SnackbarService
  ) {}

  ngOnInit() {
    this.producto = this.data;
    this.formNuevoPrecio = new FormGroup({
      nuevoPrecio: new FormControl("", [Validators.required, Validators.min(0)])
    });
  }

  ngOnDestroy() {
    this.subscriptions.unsubscribe();
  }

  public nuevoPrecio() {
    const nuevoPrecio = this.formNuevoPrecio.controls["nuevoPrecio"].value;
    const objNuevoPrecio: PrecioProducto = {
      precio: nuevoPrecio,
      fechaCreacion: this.hoy
    };
    const registrarPrecio$ = this.privateCoooperativa.registrarNuevoPrecio(
      this.producto,
      objNuevoPrecio
    );
    this.subscriptions.add(
      registrarPrecio$.subscribe(
        data => {
          this.data.preciosProductos.push(data);
          this.data.fechaModificacion = this.hoy;
          this.snackbarService.mostrar("Precio añadido.", "Ok");
          this.matDialogRef.close(data);
        },
        error => {
          this.matDialogRef.close(this.data);
          this.snackbarService.mostrar(
            "No se ha podido modificar el precio",
            "Ok",
            "snackbar-alert"
          );
        }
      )
    );
  }
}
