import { Component, OnDestroy, OnInit, ViewChild } from "@angular/core";
import { Subscription } from "rxjs";
import { Usuario } from "src/app/models/usuario";
import { PrivateAgricultorService } from "src/app/services/privateagricultor.service";
import { SnackbarService } from "src/app/services/snackbaralerts.service";
import { MatTableDataSource, MatTable } from "@angular/material/table";
import { MatDialog } from "@angular/material/dialog";
import { AddUsuarioComponent } from "../add-usuario/add-usuario.component";

@Component({
  selector: "app-ver-usuarios-agenda",
  templateUrl: "./ver-usuarios-agenda.component.html",
  styleUrls: ["./ver-usuarios-agenda.component.css"]
})
export class VerUsuariosAgendaComponent implements OnInit, OnDestroy {
  @ViewChild(MatTable, { static: true }) usuarios: MatTable<any>;
  agenda: MatTableDataSource<Usuario>;
  columnas = ["nombre", "username", "eliminar"];
  private subscriptions: Subscription = new Subscription();

  constructor(
    private privateAgricultorService: PrivateAgricultorService,
    private snackbarService: SnackbarService,
    private dialog: MatDialog
  ) {}

  ngOnInit() {
    const getAgenda$ = this.privateAgricultorService.getAgendaAgricultor();
    this.subscriptions.add(
      getAgenda$.subscribe(
        data => {
          this.agenda = new MatTableDataSource(data);
        },
        error => {
          this.agenda = new MatTableDataSource([]);
          this.snackbarService.mostrar(
            "No se ha podido recuperar los datos.",
            "Ok",
            "snackbar-alert"
          );
        }
      )
    );
  }

  public filtrar(valor: string) {
    this.agenda.filter = valor.trim().toLowerCase();
  }

  public addUsuario() {
    const dialog = this.dialog.open(AddUsuarioComponent, {
      data: { added: false },
      height: "45vh",
      width: "35vw"
    });
    const closed$ = dialog.afterClosed();
    this.subscriptions.add(
      closed$.subscribe(data => {
        if (data.added) {
          this.agenda.data.push(data.usuario);
          this.usuarios.renderRows();
        }
      })
    );
  }

  public eliminarUsuario(usuario: Usuario) {
    const eliminarUsuario$ = this.privateAgricultorService.eliminarUsuarioAgenda(
      usuario
    );
    this.subscriptions.add(
      eliminarUsuario$.subscribe(
        data => {
          const indice = this.agenda.data.indexOf(usuario);
          this.agenda.data.splice(indice, 1);
          this.usuarios.renderRows();
          this.snackbarService.mostrar("Usuario eliminado.", "Ok");
        },
        error => {
          this.snackbarService.mostrar(
            "No se ha podido eliminar el usuario.",
            "Ok",
            "snackbar-alert"
          );
        }
      )
    );
  }

  ngOnDestroy(): void {
    this.subscriptions.unsubscribe();
  }
}
