import { Injectable } from "@angular/core";
import {
  HttpInterceptor,
  HttpRequest,
  HttpHandler,
  HttpEvent,
  HttpErrorResponse,
  HttpResponse
} from "@angular/common/http";
import { Observable, throwError } from "rxjs";
import { retry, catchError, tap } from "rxjs/operators";
import { Router } from "@angular/router";
import { Constantes } from "../constantes";
import { environment } from "src/environments/environment";
import { ProgressbarService } from "./progressbarservice.service";

@Injectable()
export class HttpInterceptorService implements HttpInterceptor {
  public intercept(
    req: HttpRequest<any>,
    next: HttpHandler
  ): Observable<HttpEvent<any>> {
    this.progressbarService.mostrar.next(true);

    const url = req.url;
    const regexPrivate = new RegExp(`${environment.addToken}`);

    if (regexPrivate.test(url)) {
      req = req.clone({
        setHeaders: {
          Authorization: `${sessionStorage.getItem(
            Constantes.TOKEN_LOCAL_STORAGE
          )}`
        }
      });
    }

    return next.handle(req).pipe(
      tap(next => {
        if (next instanceof HttpResponse) {
          this.progressbarService.mostrar.next(false);
        }
      }),
      retry(2),
      catchError((error: HttpErrorResponse) => {
        this.progressbarService.mostrar.next(false);
        if (error instanceof HttpErrorResponse) {
          if (error.status === 403) {
            this.router.navigate(["/login"]);
          }
          return throwError(error);
        }
      })
    );
  }

  constructor(
    private router: Router,
    private progressbarService: ProgressbarService
  ) {}
}
