import { Injectable } from "@angular/core";
import { HttpClient } from "@angular/common/http";
import { environment } from "src/environments/environment";
import { UsuarioCooperativa } from "../models/usuario-cooperativa";
import { Observable } from "rxjs";
import { Usuario } from "../models/usuario";
import { Constantes } from "../constantes";
import { ProductoCooperativa } from "../models/producto-cooperativa";
import { UsuarioAgricultor } from "../models/usuario-agricultor";
import { PrecioProducto } from "../models/precio-producto";

@Injectable({
  providedIn: "root"
})
export class PrivateCooperativaService {
  constructor(private http: HttpClient) {}

  private url: string = `${environment.apiUrl}/private/cooperativas`;

  public getCooperativa(identificador: number): Observable<UsuarioCooperativa> {
    const endPointUrl = `${this.url}/${identificador}`;
    return this.http.get<UsuarioCooperativa>(endPointUrl);
  }

  public getProductosCooperativa(): Observable<ProductoCooperativa[]> {
    const endPointUrl = `${this.url}/${this.idCooperativaLogged}/productos`;
    return this.http.get<ProductoCooperativa[]>(endPointUrl);
  }

  public getSociosCooperativa(): Observable<UsuarioAgricultor[]> {
    const endPointUrl = `${this.url}/${this.idCooperativaLogged}/socios`;
    return this.http.get<UsuarioAgricultor[]>(endPointUrl);
  }

  public registrarNuevoPrecio(
    producto: ProductoCooperativa,
    nuevoPrecio: PrecioProducto
  ): Observable<PrecioProducto> {
    const endPointUrl = `${this.url}/${this.idCooperativaLogged}/productos/${producto.identificador}/precios`;
    return this.http.post<PrecioProducto>(endPointUrl, nuevoPrecio);
  }
  public eliminarProducto(producto: ProductoCooperativa) {
    const endPointUrl = `${this.url}/${this.idCooperativaLogged}/productos/${producto.identificador}`;
    return this.http.delete(endPointUrl);
  }

  public registrarNuevoProducto(
    nuevoProducto: ProductoCooperativa
  ): Observable<ProductoCooperativa> {
    const endPointUrl = `${this.url}/${this.idCooperativaLogged}/productos/`;
    return this.http.post<ProductoCooperativa>(endPointUrl, nuevoProducto);
  }

  private get idCooperativaLogged() {
    const coopertaiva: Usuario = JSON.parse(
      sessionStorage.getItem(Constantes.USER_LOCAL_STORAGE)
    );
    return coopertaiva.identificador;
  }
}
