import { HttpClient, HttpParams } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { Observable } from "rxjs";
import { environment } from "../../environments/environment";
import { Constantes } from "../constantes";
import { Apero } from "../models/apero";
import { Parcela } from "../models/parcela";
import { Tarea } from "../models/tarea";
import { Usuario } from "../models/usuario";
import { UsuarioAgricultor } from "../models/usuario-agricultor";

@Injectable({
  providedIn: "root"
})
export class PrivateAgricultorService {
  constructor(private http: HttpClient) {}

  private url: string = `${environment.apiUrl}/private/agricultores`;

  public getAgricultor(
    id = this.getIdUsuarioLogged()
  ): Observable<UsuarioAgricultor> {
    const endPointURL = `${this.url}/${id}`;
    return this.http.get<UsuarioAgricultor>(endPointURL);
  }

  public getTareasAgricultor(): Observable<Tarea> {
    const endPointURL: string = `${
      this.url
    }/${this.getIdUsuarioLogged()}/tareas`;
    return this.http.get<Tarea>(endPointURL);
  }

  public getAgendaAgricultor(): Observable<UsuarioAgricultor[]> {
    const endPointURL: string = `${
      this.url
    }/${this.getIdUsuarioLogged()}/registrosAgenda`;
    return this.http.get<UsuarioAgricultor[]>(endPointURL);
  }

  public getParcelasAgricultor(): Observable<Parcela[]> {
    const endPointURL: string = `${
      this.url
    }/${this.getIdUsuarioLogged()}/parcelas`;
    return this.http.get<Parcela[]>(endPointURL);
  }

  public getAperosAgricultor(
    id: number,
    tipoLabor?: number
  ): Observable<Apero[]> {
    const endPointURL: string = `${this.url}/${id}/aperos`;
    let params;
    if (tipoLabor) {
      params = new HttpParams().append("tipoLabor", `${tipoLabor}`);
    }
    return this.http.get<Apero[]>(endPointURL, { params: params });
  }

  public registrarTarea(data: Tarea): Observable<Tarea> {
    const idParcela = data.parcela.identificador;
    const endPointURL = `${
      this.url
    }/${this.getIdUsuarioLogged()}/parcelas/${idParcela}/tareas`;
    return this.http.post<Tarea>(endPointURL, data);
  }

  public eliminarTarea(tarea: Tarea) {
    const endPointURL = `${this.url}/${tarea.parcela.propietario.identificador}/parcelas/${tarea.parcela.identificador}/tareas/${tarea.identificador}`;
    return this.http.delete(endPointURL);
  }

  public modificarTarea(tarea: Tarea): Observable<Tarea> {
    const endPointURL = `${this.url}/${tarea.parcela.propietario.identificador}/parcelas/${tarea.parcela.identificador}/tareas/${tarea.identificador}`;
    return this.http.put<Tarea>(endPointURL, tarea);
  }

  public registrarApero(apero: Apero): Observable<Apero> {
    const endPointURL = `${this.url}/${this.getIdUsuarioLogged()}/aperos`;
    return this.http.post<Apero>(endPointURL, apero);
  }

  public modificarUsuario(userLogged: Usuario) {
    const endPointURL = `${this.url}/${userLogged.identificador}`;
    return this.http.put(endPointURL, userLogged);
  }

  public registrarParcela(parcela: Parcela): Observable<Parcela> {
    const propietarioId = parcela.propietario.identificador;
    const endPointURL = `${this.url}/${propietarioId}/parcelas`;
    return this.http.post<Parcela>(endPointURL, parcela);
  }

  public getParcela(identificadorParcela: number): Observable<Parcela> {
    const endPointURL = `${
      this.url
    }/${this.getIdUsuarioLogged()}/parcelas/${identificadorParcela}`;
    return this.http.get<Parcela>(endPointURL);
  }

  private getIdUsuarioLogged(): number {
    const agricultor: Usuario = JSON.parse(
      sessionStorage.getItem(Constantes.USER_LOCAL_STORAGE)
    );
    return agricultor.identificador;
  }

  public getTareasParcela(identificadorParcela: number): Observable<Tarea[]> {
    const endPointURL = `${
      this.url
    }/${this.getIdUsuarioLogged()}/parcelas/${identificadorParcela}/tareas`;
    return this.http.get<Tarea[]>(endPointURL);
  }

  public eliminarUsuarioAgenda(usuario: Usuario) {
    const endPointURL = `${
      this.url
    }/${this.getIdUsuarioLogged()}/registrosAgenda/${usuario.identificador}`;
    return this.http.delete(endPointURL);
  }

  public getAgricultores(): Observable<Usuario[]> {
    return this.http.get<Usuario[]>(this.url);
  }

  public addUsuarioAgenda(usuario: Usuario): Observable<string> {
    const endPointURL = `${
      this.url
    }/${this.getIdUsuarioLogged()}/registrosAgenda`;
    return this.http.post<string>(endPointURL, usuario);
  }
}
