import { Injectable } from "@angular/core";
import { environment } from "../../environments/environment";
import { Observable, throwError, of } from "rxjs";
import { catchError, retry } from "rxjs/operators";
import {
  HttpClient,
  HttpErrorResponse,
  HttpHeaders,
  HttpParams
} from "@angular/common/http";
import { LoginMapper } from "../models/login-mapper";

import { UsuarioAgricultor } from "../models/usuario-agricultor";
import { UsuarioCooperativa } from "../models/usuario-cooperativa";
import { Usuario } from "../models/usuario";
import { Constantes } from "../constantes";

@Injectable()
export class PublicService {
  constructor(private http: HttpClient) {}

  private END_POINT: string = `${environment.apiUrl}/public`;

  public login(username: string, password: string): Observable<any> {
    const loginEndPoint = `${this.END_POINT}/login`;
    const body = new LoginMapper(username, password);
    return this.http.post(loginEndPoint, body);
  }

  public comprobarUsername(username: string): Observable<boolean | Usuario> {
    const comprobarUsernameEndPoint = this.END_POINT;
    const userLogged: Usuario = JSON.parse(
      sessionStorage.getItem(Constantes.USER_LOCAL_STORAGE)
    );
    let params;
    if (userLogged) {
      params = new HttpParams()
        .set("username", username)
        .set("id", `${userLogged.identificador}`);
    } else {
      params = new HttpParams().set("username", username);
    }
    return this.http.get<boolean | Usuario>(comprobarUsernameEndPoint, {
      params: params
    });
  }

  public registrarAgricultor(
    iAgricultor: UsuarioAgricultor
  ): Observable<UsuarioAgricultor> {
    const registrarAgricultorEndPoint = `${this.END_POINT}/agricultores`;
    return this.http.post<UsuarioAgricultor>(
      registrarAgricultorEndPoint,
      iAgricultor
    );
  }

  public registrarCooperativa(
    iCooperativa: UsuarioCooperativa
  ): Observable<UsuarioCooperativa> {
    const registrarAgricultorEndPoint = `${this.END_POINT}/cooperativas`;
    return this.http.post<UsuarioCooperativa>(
      registrarAgricultorEndPoint,
      iCooperativa
    );
  }

  public modificarUsuario(usuario: Usuario) {
    const modificarAgricultorEndPoint = `${this.END_POINT}/agricultores/${usuario.identificador}`;
    return this.http.put<UsuarioAgricultor>(
      modificarAgricultorEndPoint,
      usuario
    );
  }
}
