import {
  AbstractControl,
  AsyncValidatorFn,
  FormGroup,
  ValidationErrors,
  FormControl,
  NG_VALIDATORS,
  Validator
} from "@angular/forms";
import { Observable, of } from "rxjs";
import { map } from "rxjs/operators";
import { Apero } from "../models/apero";
import { TipoLabor } from "../models/tipolabor";
import { Usuario } from "../models/usuario";
import { PrivateAgricultorService } from "../services/privateagricultor.service";
import { PublicService } from "../services/public.service";
import { Tarea } from "../models/tarea";
import { Directive } from "@angular/core";

export function userNameUnico(publicservice: PublicService): AsyncValidatorFn {
  return (c: AbstractControl): Observable<ValidationErrors | null> => {
    const username = c.value;
    return publicservice.comprobarUsername(username).pipe(
      map(usernameUsado => {
        return !usernameUsado ? null : { userNameUnicoValidator: true };
      })
    );
  };
}

export function agricultorAptoTareaValidator(
  privateAgricultorService: PrivateAgricultorService
): AsyncValidatorFn {
  return (fg: FormGroup): Observable<ValidationErrors | null> => {
    const tipoLabor: TipoLabor = fg.controls["tipoLabor"].value;
    const encargado: Usuario = fg.controls["encargado"].value;
    return privateAgricultorService
      .getAperosAgricultor(encargado.identificador, tipoLabor.identificador)
      .pipe(
        map(repuesta => {
          return repuesta.length > 0
            ? null
            : { agricultorAptoTareaValidator: true };
        })
      );
  };
}

export function aperoAptoTareaValidator(tarea: Tarea): AsyncValidatorFn {
  return (fc: FormControl): Observable<ValidationErrors | null> => {
    const tipoLabor = tarea.tipoLabor;
    const apero: Apero = fc.value;
    let iguales = false;
    if (apero.tipoLabores) {
      apero.tipoLabores.forEach(tl => {
        if (tl.identificador === tipoLabor.identificador) {
          iguales = true;
        }
      });
      return of(iguales ? null : { aperoAptoTareaValidator: true });
    }

    return of({ aperoAptoTareaValidator: true });
  };
}

export function opcionSeleccionadaValidator(): AsyncValidatorFn {
  return (fc: FormControl): Observable<ValidationErrors | null> => {
    const valor: any = fc.value;
    return typeof valor === "string"
      ? of({ opcionSeleccionadaValidator: true })
      : of(null);
  };
}
export function repetirPasswordCorrectoValidator(): AsyncValidatorFn {
  return (fg: FormGroup): Observable<ValidationErrors | null> => {
    const passwordOriginal = fg.controls["password"].value;
    const passwordRepetida = fg.controls["passwordRepetida"].value;
    return passwordOriginal === passwordRepetida
      ? of(null)
      : of({ repetirPasswordCorrectoValidator: true });
  };
}
