package com.tfg.gestiontareas.api;

import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import com.fasterxml.jackson.annotation.JsonView;
import com.tfg.gestiontareas.model.Apero;
import com.tfg.gestiontareas.model.EnumEstadoTarea;
import com.tfg.gestiontareas.model.EnumRole;
import com.tfg.gestiontareas.model.Parcela;
import com.tfg.gestiontareas.model.Tarea;
import com.tfg.gestiontareas.model.TipoLabor;
import com.tfg.gestiontareas.model.Usuario;
import com.tfg.gestiontareas.model.UsuarioAgricultor;
import com.tfg.gestiontareas.repository.IRepoApero;
import com.tfg.gestiontareas.repository.IRepoParcela;
import com.tfg.gestiontareas.repository.IRepoProductoTareaEmpleado;
import com.tfg.gestiontareas.repository.IRepoProductoTareaRecogido;
import com.tfg.gestiontareas.repository.IRepoRegistroHoras;
import com.tfg.gestiontareas.repository.IRepoTarea;
import com.tfg.gestiontareas.repository.IRepoTipoLabor;
import com.tfg.gestiontareas.repository.IRepoUsuario;
import com.tfg.gestiontareas.util.Views;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/private/agricultores")
public class PrivateAgricultoresController {

    private Logger logger = LoggerFactory.getLogger(PrivateAgricultoresController.class);

    @Autowired
    private IRepoUsuario iRepoUsuario;

    @Autowired
    private IRepoApero iRepoApero;

    @Autowired
    private IRepoParcela iRepoParcela;

    @Autowired
    private IRepoTarea iRepoTarea;

    @Autowired
    private IRepoProductoTareaEmpleado iRepoProductoTareaEmpleado;

    @Autowired
    private IRepoProductoTareaRecogido iRepoProductoTareaRecogido;

    @Autowired
    private IRepoRegistroHoras iRepoRegistrosHoras;

    @Autowired
    private IRepoTipoLabor iRepoTipoLabor;

    @JsonView(Views.ListView.class)
    @GetMapping(value = "")
    public ResponseEntity<List<Usuario>> getTodos() {
        List<Usuario> usuarios = iRepoUsuario.findByRol(EnumRole.ROLE_AGRICULTOR);
        return new ResponseEntity<List<Usuario>>(usuarios, HttpStatus.OK);
    }

    @GetMapping(value = "/{id}")
    public ResponseEntity<?> getUsuario(@PathVariable Long id) {
        try {
            UsuarioAgricultor userAgri = getAgricultor(id);
            if (userAgri != null) {
                return new ResponseEntity<UsuarioAgricultor>(userAgri, HttpStatus.OK);
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        }
    }

    @PutMapping(value = "/{id}")
    public ResponseEntity<?> modificarAgricultor(@PathVariable Long id, @RequestBody UsuarioAgricultor entity) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            if (usuarioAgricultor != null) {
                if (isValido(entity, usuarioAgricultor)) {
                    usuarioAgricultor.setUsername(entity.getUsername());
                    usuarioAgricultor.setMunicipio(entity.getMunicipio());
                    usuarioAgricultor.setNombre(entity.getNombre());
                    usuarioAgricultor.setCooperativa(entity.getCooperativa());
                    Usuario registrado = iRepoUsuario.save(usuarioAgricultor);
                    return new ResponseEntity<Usuario>(registrado, HttpStatus.OK);
                } else {
                    return new ResponseEntity<String>(Constantes.YA_EXISTE_UN_USUARIO_CON_ESE_CORREO_ELECTRÓNICO,
                            HttpStatus.BAD_REQUEST);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (DataIntegrityViolationException divex) {
            logger.error(divex.getMessage());
            return new ResponseEntity<String>(Constantes.ALGUNO_DE_LOS_CAMPOS_OBLIGATORIOS_ES_NULO,
                    HttpStatus.BAD_REQUEST);
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @JsonView(Views.ListView.class)
    @GetMapping(value = "/{id}/aperos")
    public ResponseEntity<?> getAperos(@PathVariable Long id,
            @RequestParam(required = false, name = "tipoLabor") Long idTipoLabor) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            if (usuarioAgricultor != null) {
                if (idTipoLabor == null) {
                    List<Apero> aperosUsuario = iRepoApero.findByPropietario(usuarioAgricultor);
                    return new ResponseEntity<List<Apero>>(aperosUsuario, HttpStatus.OK);
                } else {
                    Optional<TipoLabor> optTipoLaor = iRepoTipoLabor.findById(idTipoLabor);
                    if (optTipoLaor.isPresent()) {
                        List<Apero> aperosUsuario = iRepoApero
                                .findByPropietarioAndTipoLaboresContaining(usuarioAgricultor, optTipoLaor.get());
                        return new ResponseEntity<List<Apero>>(aperosUsuario, HttpStatus.OK);
                    } else {
                        return new ResponseEntity<String>(Constantes.TIPO_LABOR_NO_ENCONTRADA, HttpStatus.NOT_FOUND);
                    }

                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (DataIntegrityViolationException divex) {
            return new ResponseEntity<String>(Constantes.ALGUNO_DE_LOS_CAMPOS_OBLIGATORIOS_ES_NULO,
                    HttpStatus.BAD_REQUEST);
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PostMapping(value = "/{id}/aperos")
    public ResponseEntity<?> registrarNuevoApero(@RequestBody Apero entity, @PathVariable Long id) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            if (usuarioAgricultor != null) {
                usuarioAgricultor.registrarApero(entity);
                Apero registrado = iRepoApero.save(entity);
                return new ResponseEntity<Apero>(registrado, HttpStatus.CREATED);
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (DataIntegrityViolationException divex) {
            logger.error(divex.getMessage());
            return new ResponseEntity<String>(Constantes.ALGUNO_DE_LOS_CAMPOS_OBLIGATORIOS_ES_NULO,
                    HttpStatus.BAD_REQUEST);
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @GetMapping(value = "/{id}/aperos/{idApero}")
    public ResponseEntity<?> getApero(@PathVariable Long id, @PathVariable Long idApero) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            if (usuarioAgricultor != null) {
                Optional<Apero> optApero = iRepoApero.findByIdentificadorAndPropietario(idApero, usuarioAgricultor);
                if (optApero.isPresent()) {
                    return new ResponseEntity<Apero>(optApero.get(), HttpStatus.OK);
                } else {
                    return new ResponseEntity<String>(Constantes.APERO_NO_ENCONTRADO, HttpStatus.NOT_FOUND);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.BAD_REQUEST);
        }
    }

    @JsonView(Views.ListView.class)
    @GetMapping(value = "/{id}/parcelas")
    public ResponseEntity<?> getParcelas(@PathVariable Long id) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            if (usuarioAgricultor != null) {
                List<Parcela> parcelas = iRepoParcela.findByPropietario(usuarioAgricultor);
                return new ResponseEntity<List<Parcela>>(parcelas, HttpStatus.OK);
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PostMapping(value = "/{id}/parcelas")
    public ResponseEntity<?> registrarNuevaParcela(@RequestBody Parcela entity, @PathVariable Long id) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            if (usuarioAgricultor != null) {
                usuarioAgricultor.registrarParcela(entity);
                Parcela registrada = iRepoParcela.save(entity);
                return new ResponseEntity<Parcela>(registrada, HttpStatus.CREATED);
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (DataIntegrityViolationException divex) {
            logger.error(divex.getMessage());
            return new ResponseEntity<String>(Constantes.ALGUNO_DE_LOS_CAMPOS_OBLIGATORIOS_ES_NULO,
                    HttpStatus.BAD_REQUEST);
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @GetMapping(value = "/{id}/parcelas/{idParcela}")
    public ResponseEntity<?> getParcela(@PathVariable Long id, @PathVariable Long idParcela) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            if (usuarioAgricultor != null) {
                Optional<Parcela> optParcela = iRepoParcela.findByIdentificadorAndPropietario(idParcela,
                        usuarioAgricultor);
                if (optParcela.isPresent()) {
                    return new ResponseEntity<Parcela>(optParcela.get(), HttpStatus.OK);
                } else {
                    return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_LA_PARCELA, HttpStatus.NOT_FOUND);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.NOT_FOUND);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @GetMapping(value = "/{id}/registrosAgenda")
    public ResponseEntity<?> getRegistrosAgenda(@PathVariable Long id) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            if (usuarioAgricultor != null) {
                Set<UsuarioAgricultor> agenda = usuarioAgricultor.getAgenda();
                for (UsuarioAgricultor t : usuarioAgricultor.getEnAgendaDe()) {
                    if (!agenda.contains(t)) {
                        agenda.add(t);
                    }
                }
                return new ResponseEntity<Set<UsuarioAgricultor>>(agenda, HttpStatus.OK);
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PostMapping(value = "/{id}/registrosAgenda")
    public ResponseEntity<?> crearRegistroAgenda(@RequestBody Usuario entity, @PathVariable Long id) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            UsuarioAgricultor usuarioAgricultorB = getAgricultor(entity.getIdentificador());
            if (usuarioAgricultor != null && usuarioAgricultorB != null) {
                boolean enAgenda = isUsuarioEnAgenda(usuarioAgricultor, usuarioAgricultorB);
                if (!enAgenda) {
                    usuarioAgricultor.getAgenda().add(usuarioAgricultorB);
                    usuarioAgricultorB.getEnAgendaDe().add(usuarioAgricultor);
                    iRepoUsuario.save(usuarioAgricultor);
                    iRepoUsuario.save(usuarioAgricultorB);
                    return ResponseEntity.noContent().build();
                } else {
                    return new ResponseEntity<String>(Constantes.EL_USUARIO_YA_ESTABA_EN_LA_AGENDA,
                            HttpStatus.BAD_REQUEST);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @DeleteMapping(value = "/{id}/registrosAgenda/{idUsuario}")
    public ResponseEntity<?> eliminarRegistroAgenda(@PathVariable Long id, @PathVariable Long idUsuario) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            UsuarioAgricultor usuarioAgricultorB = getAgricultor(idUsuario);
            if (usuarioAgricultor != null && usuarioAgricultorB != null) {
                boolean enAgenda = isUsuarioEnAgenda(usuarioAgricultor, usuarioAgricultorB);
                if (enAgenda) {
                    if (eliminarUsuarioAgenda(usuarioAgricultor, usuarioAgricultorB)) {
                        iRepoUsuario.save(usuarioAgricultor);
                        iRepoUsuario.save(usuarioAgricultorB);
                        return ResponseEntity.noContent().build();
                    } else {
                        return new ResponseEntity<String>("No se ha podido eliminar el usuario.",
                                HttpStatus.INTERNAL_SERVER_ERROR);
                    }
                } else {
                    boolean enAgendaDe = isUsuarioEnAgendaDe(usuarioAgricultor, usuarioAgricultorB);
                    if (enAgendaDe) {
                        if (eliminarUsuarioAgenda(usuarioAgricultorB, usuarioAgricultor)) {
                            iRepoUsuario.save(usuarioAgricultor);
                            iRepoUsuario.save(usuarioAgricultorB);
                            return ResponseEntity.noContent().build();
                        } else {
                            return new ResponseEntity<String>("No se ha podido eliminar el usuario.",
                                    HttpStatus.INTERNAL_SERVER_ERROR);
                        }
                    }
                    return new ResponseEntity<String>(Constantes.EL_USUARIO_NO_ESTABA_EN_LA_AGENDA,
                            HttpStatus.BAD_REQUEST);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.BAD_REQUEST);
            }
        } catch (

        IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    private boolean isUsuarioEnAgenda(UsuarioAgricultor usuarioAgricultor, UsuarioAgricultor usuarioAgricultorb) {
        return usuarioAgricultor.getIdentificador().equals(usuarioAgricultorb.getIdentificador())
                || usuarioAgricultor.getAgenda().stream()
                        .filter(usuario -> usuario.getIdentificador().equals(usuarioAgricultorb.getIdentificador()))
                        .collect(Collectors.toList()).size() > 0;
    }

    private boolean isUsuarioEnAgendaDe(UsuarioAgricultor usuarioAgricultor, UsuarioAgricultor usuarioAgricultorb) {
        return usuarioAgricultor.getIdentificador().equals(usuarioAgricultorb.getIdentificador())
                || usuarioAgricultor.getEnAgendaDe().stream()
                        .filter(usuario -> usuario.getIdentificador().equals(usuarioAgricultorb.getIdentificador()))
                        .collect(Collectors.toList()).size() > 0;
    }

    private boolean eliminarUsuarioAgenda(UsuarioAgricultor a, UsuarioAgricultor b) {
        boolean eliminadoAgenda = false;
        boolean eliminadoEnAgendaDe = false;
        for (UsuarioAgricultor user : a.getAgenda()) {
            if (user.getIdentificador().equals(b.getIdentificador())) {
                eliminadoAgenda = a.getAgenda().remove(user);
                break;
            }
        }
        for (UsuarioAgricultor user : b.getEnAgendaDe()) {
            if (user.getIdentificador().equals(a.getIdentificador())) {
                eliminadoEnAgendaDe = b.getEnAgendaDe().remove(user);
                break;
            }
        }

        return eliminadoAgenda && eliminadoEnAgendaDe;
    }

    private boolean isValido(UsuarioAgricultor entity, UsuarioAgricultor usuarioAgricultor) {
        Optional<Usuario> optUsuario = iRepoUsuario.findByUsername(entity.getUsername());
        if (optUsuario.isPresent()) {
            Usuario usuario = optUsuario.get();
            if (usuario.getIdentificador() != usuarioAgricultor.getIdentificador()) {
                return false;
            } else {
                return true;
            }
        } else {
            return true;
        }
    }

    private UsuarioAgricultor getAgricultor(Long id) throws IllegalArgumentException {
        Optional<Usuario> optUsuario = iRepoUsuario.findById(id);
        if (optUsuario.isPresent()) {
            Usuario usuario = optUsuario.get();
            if (usuario instanceof UsuarioAgricultor) {
                return (UsuarioAgricultor) usuario;
            } else {
                throw new IllegalArgumentException(Constantes.ESTE_USUARIO_NO_ES_UN_AGRICULTOR);
            }
        } else {
            return null;
        }
    }

}