package com.tfg.gestiontareas.api;

import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.Set;

import com.fasterxml.jackson.annotation.JsonView;
import com.tfg.gestiontareas.model.EnumRole;
import com.tfg.gestiontareas.model.PrecioProducto;
import com.tfg.gestiontareas.model.ProductoCooperativa;
import com.tfg.gestiontareas.model.Usuario;
import com.tfg.gestiontareas.model.UsuarioAgricultor;
import com.tfg.gestiontareas.model.UsuarioCooperativa;
import com.tfg.gestiontareas.repository.IRepoPrecioProducto;
import com.tfg.gestiontareas.repository.IRepoProductoCooperativa;
import com.tfg.gestiontareas.repository.IRepoUsuario;
import com.tfg.gestiontareas.util.Views;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping(value = "/private/cooperativas")
public class PrivateCooperativaController {

    private Logger logger = LoggerFactory.getLogger(PrivateCooperativaController.class);

    @Autowired
    private IRepoUsuario iRepoUsuario;

    @Autowired
    private IRepoProductoCooperativa iRepoProductoCooperativa;

    @Autowired
    private IRepoPrecioProducto iRepoPrecioProducto;

    @GetMapping
    public ResponseEntity<List<Usuario>> getCooperativas() {
        List<Usuario> cooperativas = iRepoUsuario.findByRol(EnumRole.ROLE_RESPONSABLE_COOPERATIVA);
        return new ResponseEntity<List<Usuario>>(cooperativas, HttpStatus.OK);
    }

    @GetMapping(value = "/{id}")
    public ResponseEntity<?> getCooperativa(@PathVariable Long id) {
        try {
            UsuarioCooperativa userAgri = _getCooperativa(id);
            if (userAgri != null) {
                return new ResponseEntity<UsuarioCooperativa>(userAgri, HttpStatus.OK);
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage(), iaex);
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @GetMapping(value = "/{id}/productos")
    public ResponseEntity<?> getProductos(@PathVariable Long id) {
        try {
            UsuarioCooperativa userCooperativa = _getCooperativa(id);
            if (userCooperativa != null) {
                List<ProductoCooperativa> productosCooperativa = iRepoProductoCooperativa
                        .findByCooperativa(userCooperativa);
                return new ResponseEntity<List<ProductoCooperativa>>(productosCooperativa, HttpStatus.OK);
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage(), iaex);
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PostMapping(value = "/{id}/productos")
    public ResponseEntity<?> registrarProducto(@PathVariable Long id, @RequestBody ProductoCooperativa entity) {
        try {
            UsuarioCooperativa userCooperativa = _getCooperativa(id);
            if (userCooperativa != null) {
                entity.setFechaModificacion(new Date());
                userCooperativa.registarProductoCooperativa(entity);
                ProductoCooperativa guardado = iRepoProductoCooperativa.save(entity);
                if (entity.getPreciosProductos() != null && entity.getPreciosProductos().size() > 0) {
                    guardado.registrarPrecioProducto(entity.getPreciosProductos());
                    iRepoPrecioProducto.saveAll(entity.getPreciosProductos());
                    return new ResponseEntity<ProductoCooperativa>(guardado, HttpStatus.CREATED);
                } else {
                    return new ResponseEntity<String>(Constantes.ALGUNO_DE_LOS_CAMPOS_OBLIGATORIOS_ES_NULO,
                            HttpStatus.BAD_REQUEST);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (DataIntegrityViolationException divex) {
            logger.error(divex.getMessage());
            return new ResponseEntity<String>(Constantes.ALGUNO_DE_LOS_CAMPOS_OBLIGATORIOS_ES_NULO,
                    HttpStatus.BAD_REQUEST);
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage(), iaex);
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @GetMapping(value = "/{id}/productos/{idProducto}")
    public ResponseEntity<?> getProducto(@PathVariable Long id, @PathVariable Long idProducto) {
        try {
            UsuarioCooperativa userCooperativa = _getCooperativa(id);
            if (userCooperativa != null) {
                Optional<ProductoCooperativa> optProductoCooperativa = iRepoProductoCooperativa
                        .findByIdentificadorAndCooperativa(idProducto, userCooperativa);
                if (optProductoCooperativa.isPresent()) {
                    return new ResponseEntity<ProductoCooperativa>(optProductoCooperativa.get(), HttpStatus.OK);
                } else {
                    return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_PRODUCTO, HttpStatus.NOT_FOUND);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage(), iaex);
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PutMapping(value = "/{id}/productos/{idProducto}")
    public ResponseEntity<?> modificarProducto(@PathVariable Long id, @PathVariable Long idProducto,
            @RequestBody ProductoCooperativa entity) {
        try {
            UsuarioCooperativa userCooperativa = _getCooperativa(id);
            if (userCooperativa != null) {
                Optional<ProductoCooperativa> optProductoCooperativa = iRepoProductoCooperativa
                        .findByIdentificadorAndCooperativa(idProducto, userCooperativa);
                if (optProductoCooperativa.isPresent()) {
                    ProductoCooperativa pCooperativa = optProductoCooperativa.get();
                    pCooperativa.setNombre(entity.getNombre());
                    pCooperativa.setFechaModificacion(new Date());
                    return new ResponseEntity<ProductoCooperativa>(iRepoProductoCooperativa.save(pCooperativa),
                            HttpStatus.OK);
                } else {
                    return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.NOT_FOUND);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (DataIntegrityViolationException divex) {
            logger.error(divex.getMessage());
            return new ResponseEntity<String>(Constantes.ALGUNO_DE_LOS_CAMPOS_OBLIGATORIOS_ES_NULO,
                    HttpStatus.BAD_REQUEST);
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage(), iaex);
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @DeleteMapping(value = "/{id}/productos/{idProducto}")
    public ResponseEntity<?> eliminarProducto(@PathVariable Long id, @PathVariable Long idProducto) {
        try {
            UsuarioCooperativa userCooperativa = _getCooperativa(id);
            if (userCooperativa != null) {
                Optional<ProductoCooperativa> optProductoCooperativa = iRepoProductoCooperativa
                        .findByIdentificadorAndCooperativa(idProducto, userCooperativa);
                if (optProductoCooperativa.isPresent()) {
                    iRepoProductoCooperativa.delete(optProductoCooperativa.get());
                    return ResponseEntity.noContent().build();
                } else {
                    return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_PRODUCTO, HttpStatus.NOT_FOUND);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage(), iaex);
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @GetMapping(value = "/{id}/productos/{idProducto}/precios")
    public ResponseEntity<?> getPreciosProducto(@PathVariable Long id, @PathVariable Long idProducto) {
        try {
            UsuarioCooperativa userCooperativa = _getCooperativa(id);
            if (userCooperativa != null) {
                Optional<ProductoCooperativa> optProductoCooperativa = iRepoProductoCooperativa
                        .findByIdentificadorAndCooperativa(idProducto, userCooperativa);
                if (optProductoCooperativa.isPresent()) {
                    List<PrecioProducto> preciosProductoCooperativa = iRepoPrecioProducto
                            .findByProductoCooperativa(optProductoCooperativa.get());
                    return new ResponseEntity<List<PrecioProducto>>(preciosProductoCooperativa, HttpStatus.OK);
                } else {
                    return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_PRODUCTO, HttpStatus.NOT_FOUND);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage(), iaex);
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PostMapping(value = "/{id}/productos/{idProducto}/precios")
    public ResponseEntity<?> crearPrecioProducto(@PathVariable Long id, @PathVariable Long idProducto,
            @RequestBody PrecioProducto entity) {
        try {
            UsuarioCooperativa userCooperativa = _getCooperativa(id);
            if (userCooperativa != null) {
                Optional<ProductoCooperativa> optProductoCooperativa = iRepoProductoCooperativa
                        .findByIdentificadorAndCooperativa(idProducto, userCooperativa);
                if (optProductoCooperativa.isPresent()) {
                    ProductoCooperativa pCooperativa = optProductoCooperativa.get();
                    entity.setFechaCreacion(new Date());
                    pCooperativa.registrarPrecioProducto(entity);
                    pCooperativa.setFechaModificacion(new Date());
                    iRepoProductoCooperativa.save(pCooperativa);
                    return new ResponseEntity<PrecioProducto>(iRepoPrecioProducto.save(entity), HttpStatus.CREATED);
                } else {
                    return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_PRODUCTO, HttpStatus.NOT_FOUND);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (DataIntegrityViolationException divex) {
            logger.error(divex.getMessage());
            return new ResponseEntity<String>(Constantes.ALGUNO_DE_LOS_CAMPOS_OBLIGATORIOS_ES_NULO,
                    HttpStatus.BAD_REQUEST);
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage(), iaex);
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    private UsuarioCooperativa _getCooperativa(Long id) throws IllegalArgumentException {
        Optional<Usuario> optUsuario = iRepoUsuario.findById(id);
        if (optUsuario.isPresent()) {
            Usuario usuario = optUsuario.get();
            if (usuario instanceof UsuarioCooperativa) {
                return (UsuarioCooperativa) usuario;
            } else {
                throw new IllegalArgumentException(Constantes.ESTE_USUARIO_NO_ES_UNA_COOPERATIVA);
            }
        } else {
            return null;
        }
    }

}