package com.tfg.gestiontareas.api;

import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

import com.tfg.gestiontareas.model.Apero;
import com.tfg.gestiontareas.model.EnumEstadoTarea;
import com.tfg.gestiontareas.model.Parcela;
import com.tfg.gestiontareas.model.Tarea;
import com.tfg.gestiontareas.model.Usuario;
import com.tfg.gestiontareas.model.UsuarioAgricultor;
import com.tfg.gestiontareas.repository.IRepoParcela;
import com.tfg.gestiontareas.repository.IRepoProductoTareaEmpleado;
import com.tfg.gestiontareas.repository.IRepoProductoTareaRecogido;
import com.tfg.gestiontareas.repository.IRepoRegistroHoras;
import com.tfg.gestiontareas.repository.IRepoTarea;
import com.tfg.gestiontareas.repository.IRepoUsuario;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/private/agricultores/{id}")
public class PrivateTareasController {

    private Logger logger = LoggerFactory.getLogger(PrivateTareasController.class);

    @Autowired
    private IRepoTarea iRepoTarea;

    @Autowired
    private IRepoProductoTareaEmpleado iRepoProductoTareaEmpleado;

    @Autowired
    private IRepoProductoTareaRecogido iRepoProductoTareaRecogido;

    @Autowired
    private IRepoRegistroHoras iRepoRegistrosHoras;

    @Autowired
    private IRepoParcela iRepoParcela;

    @Autowired
    private IRepoUsuario iRepoUsuario;

    @GetMapping(value = "/parcelas/{idParcela}/tareas")
    public ResponseEntity<?> getTareas(@PathVariable Long id, @PathVariable Long idParcela) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            if (usuarioAgricultor != null) {
                Optional<Parcela> optParcela = iRepoParcela.findByIdentificadorAndPropietario(idParcela,
                        usuarioAgricultor);
                if (optParcela.isPresent()) {
                    return new ResponseEntity<Set<Tarea>>(optParcela.get().getTareas(), HttpStatus.OK);

                } else {
                    return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_LA_PARCELA, HttpStatus.NOT_FOUND);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PostMapping(value = "/parcelas/{idParcela}/tareas")
    public ResponseEntity<?> crearTarea(@RequestBody Tarea entity, @PathVariable Long id,
            @PathVariable Long idParcela) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            if (usuarioAgricultor != null) {
                Optional<Parcela> optParcela = iRepoParcela.findByIdentificadorAndPropietario(idParcela,
                        usuarioAgricultor);
                if (optParcela.isPresent()) {
                    Parcela p = optParcela.get();
                    p.registrarTarea(entity);
                    Tarea guardada;
                    if (entity.getTipoLabor().isNecesitaProducto()) {
                        if (entity.getProductosTareaEmpleados() != null
                                && entity.getProductosTareaEmpleados().size() > 0) {
                            guardada = iRepoTarea.save(entity);
                            guardada.registarProductoTareaEmpleado(entity.getProductosTareaEmpleados());
                            iRepoProductoTareaEmpleado.saveAll(entity.getProductosTareaEmpleados());
                            return new ResponseEntity<Tarea>(guardada, HttpStatus.CREATED);
                        } else {
                            return new ResponseEntity<String>(Constantes.HAY_QUE_INDICAR_PRODUCTO_EMPLEADO,
                                    HttpStatus.BAD_REQUEST);
                        }
                    } else {
                        guardada = iRepoTarea.save(entity);
                        return new ResponseEntity<Tarea>(guardada, HttpStatus.CREATED);
                    }
                } else {
                    return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_LA_PARCELA, HttpStatus.NOT_FOUND);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (DataIntegrityViolationException divex) {
            logger.error(divex.getMessage());
            return new ResponseEntity<String>(Constantes.ALGUNO_DE_LOS_CAMPOS_OBLIGATORIOS_ES_NULO,
                    HttpStatus.BAD_REQUEST);
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @GetMapping(value = "/parcelas/{idParcela}/tareas/{idTarea}")
    public ResponseEntity<?> getTarea(@PathVariable Long id, @PathVariable Long idParcela, @PathVariable Long idTarea) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            if (usuarioAgricultor != null) {
                Optional<Parcela> optParcela = iRepoParcela.findByIdentificadorAndPropietario(idParcela,
                        usuarioAgricultor);
                if (optParcela.isPresent()) {
                    Optional<Tarea> optTarea = iRepoTarea.findByIdentificadorAndParcela(idTarea, optParcela.get());
                    if (optTarea.isPresent()) {
                        return new ResponseEntity<Tarea>(optTarea.get(), HttpStatus.OK);
                    } else {
                        return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_LA_TAREA,
                                HttpStatus.NOT_FOUND);
                    }
                } else {
                    return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_LA_PARCELA, HttpStatus.NOT_FOUND);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @DeleteMapping(value = "/parcelas/{idParcela}/tareas/{idTarea}")
    public ResponseEntity<?> eliminarTarea(@PathVariable Long id, @PathVariable Long idParcela,
            @PathVariable Long idTarea) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            if (usuarioAgricultor != null) {
                Optional<Parcela> optParcela = iRepoParcela.findByIdentificadorAndPropietario(idParcela,
                        usuarioAgricultor);
                if (optParcela.isPresent()) {
                    Optional<Tarea> optTarea = iRepoTarea.findByIdentificadorAndParcela(idTarea, optParcela.get());
                    if (optTarea.isPresent()) {
                        Tarea tarea = optTarea.get();
                        if (!(tarea.getEstadoTarea().equals(EnumEstadoTarea.PTE_ESTIMACION))) {
                            if (tarea.getEstadoTarea().equals(EnumEstadoTarea.COMPLETADA)) {
                                return new ResponseEntity<String>(Constantes.NO_SE_PUEDE_ELIMINAR_ESTA_TAREA,
                                        HttpStatus.BAD_REQUEST);
                            } else {
                                double diffDias = diferenciaEntreFechasDias(tarea);
                                if (diffDias < Tarea.DIAS_PARA_ELIMINAR_TAREA) {
                                    return new ResponseEntity<String>(Constantes.ELIMINAR_TAREAS_MENOS_DIAS,
                                            HttpStatus.BAD_REQUEST);
                                } else {
                                    iRepoTarea.delete(tarea);
                                    return ResponseEntity.noContent().build();
                                }
                            }
                        } else {
                            iRepoTarea.delete(tarea);
                            return ResponseEntity.noContent().build();
                        }
                    } else {
                        return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_LA_TAREA,
                                HttpStatus.NOT_FOUND);
                    }
                } else {
                    return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_LA_PARCELA, HttpStatus.NOT_FOUND);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @PutMapping(value = "/parcelas/{idParcela}/tareas/{idTarea}")
    public ResponseEntity<?> modificarTarea(@PathVariable Long id, @PathVariable Long idParcela,
            @PathVariable Long idTarea, @RequestBody Tarea entity) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            if (usuarioAgricultor != null) {
                Optional<Parcela> optParcela = iRepoParcela.findByIdentificadorAndPropietario(idParcela,
                        usuarioAgricultor);
                if (optParcela.isPresent()) {
                    Optional<Tarea> optTarea = iRepoTarea.findByIdentificadorAndParcela(idTarea, optParcela.get());
                    if (optTarea.isPresent()) {
                        Tarea guardada;
                        if (entity.getEstadoTarea().equals(EnumEstadoTarea.ESTIMADA)) {
                            guardada = _modificarTareaAEstimada(optTarea.get(), entity);
                        } else {
                            guardada = _modificarEstadoACompletada(optTarea.get(), entity);
                        }
                        return new ResponseEntity<Tarea>(iRepoTarea.save(guardada), HttpStatus.OK);
                    } else {
                        return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_LA_TAREA,
                                HttpStatus.NOT_FOUND);
                    }
                } else {
                    return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_LA_PARCELA, HttpStatus.NOT_FOUND);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalStateException isex) {
            logger.error(isex.getMessage());
            return new ResponseEntity<String>(isex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @GetMapping(value = "/tareas")
    public ResponseEntity<?> getTareas(@PathVariable Long id,
            @RequestParam(name = "hoy", required = false, defaultValue = "false") Boolean hoy) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            List<Tarea> tareas = iRepoTarea.findByEncargado(usuarioAgricultor);
            for (Tarea t : iRepoTarea.findByParcelaPropietario(usuarioAgricultor)) {
                if (!tareas.contains(t)) {
                    tareas.add(t);
                }
            }
            if (hoy) {
                tareas = tareas.stream().filter(tarea -> tarea.getEstadoTarea().equals(EnumEstadoTarea.ESTIMADA)
                        && Math.floor(diferenciaEntreFechasDias(tarea)) == 0).collect(Collectors.toList());
            }
            return new ResponseEntity<List<Tarea>>(tareas, HttpStatus.OK);
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @DeleteMapping(value = "/tareas/{idTarea}")
    public ResponseEntity<?> eliminarTarea(@PathVariable Long id, @PathVariable Long idTarea) {
        try {
            UsuarioAgricultor usuarioAgricultor = getAgricultor(id);
            if (usuarioAgricultor != null) {
                Optional<Tarea> optTarea = iRepoTarea.findById(idTarea);
                if (optTarea.isPresent()) {
                    Tarea tarea = optTarea.get();
                    if (!(tarea.getEstadoTarea().equals(EnumEstadoTarea.PTE_ESTIMACION))) {
                        if (tarea.getEstadoTarea().equals(EnumEstadoTarea.COMPLETADA)) {
                            return new ResponseEntity<String>(Constantes.NO_SE_PUEDE_ELIMINAR_ESTA_TAREA,
                                    HttpStatus.BAD_REQUEST);
                        } else {
                            double diffDias = diferenciaEntreFechasDias(tarea);
                            if (diffDias < Tarea.DIAS_PARA_ELIMINAR_TAREA) {
                                return new ResponseEntity<String>(Constantes.NO_SE_PUEDE_ELIMINAR_ESTA_TAREA,
                                        HttpStatus.BAD_REQUEST);
                            } else {
                                iRepoTarea.delete(tarea);
                                return ResponseEntity.noContent().build();
                            }
                        }
                    } else {
                        iRepoTarea.delete(tarea);
                        return ResponseEntity.noContent().build();
                    }
                } else {
                    return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_LA_TAREA, HttpStatus.NOT_FOUND);
                }
            } else {
                return new ResponseEntity<String>(Constantes.NO_SE_HA_ENCONTRADO_EL_USUARIO, HttpStatus.NOT_FOUND);
            }
        } catch (IllegalArgumentException iaex) {
            logger.error(iaex.getMessage());
            return new ResponseEntity<String>(iaex.getMessage(), HttpStatus.BAD_REQUEST);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(Constantes.ERROR_EN_EL_SERVIDOR, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    private UsuarioAgricultor getAgricultor(Long id) throws IllegalArgumentException {
        Optional<Usuario> optUsuario = iRepoUsuario.findById(id);
        if (optUsuario.isPresent()) {
            Usuario usuario = optUsuario.get();
            if (usuario instanceof UsuarioAgricultor) {
                return (UsuarioAgricultor) usuario;
            } else {
                throw new IllegalArgumentException(Constantes.ESTE_USUARIO_NO_ES_UN_AGRICULTOR);
            }
        } else {
            return null;
        }
    }

    private Tarea _modificarTareaAEstimada(Tarea tarea, Tarea entity) throws IllegalStateException {
        Date fechaInicio = entity.getFechaInicio();
        Float tiempoEstimado = entity.getTiempoEstimadoHoras();
        Apero apero = entity.getApero();
        if (fechaInicio != null && tiempoEstimado != null && apero != null) {
            tarea.modificarEstadoTarea(entity.getEstadoTarea());
            tarea.setFechaInicio(entity.getFechaInicio());
            tarea.setTiempoEstimadoHoras(entity.getTiempoEstimadoHoras());
            tarea.modificarAperoTarea(entity.getApero());
            if (tarea.getTipoLabor().isNecesitaProducto()) {
                if (entity.getProductosTareaEmpleados() != null && entity.getProductosTareaEmpleados().size() > 0) {
                    tarea.registarProductoTareaEmpleado(entity.getProductosTareaEmpleados());
                    iRepoProductoTareaEmpleado.saveAll(entity.getProductosTareaEmpleados());
                } else {
                    throw new IllegalArgumentException(Constantes.HAY_QUE_INDICAR_PRODUCTO_EMPLEADO);
                }
            }
            return tarea;
        } else {
            throw new IllegalArgumentException(Constantes.ALGUNO_DE_LOS_CAMPOS_OBLIGATORIOS_ES_NULO);
        }
    }

    private Tarea _modificarEstadoACompletada(Tarea tarea, Tarea entity) throws IllegalStateException {
        tarea.modificarEstadoTarea(entity.getEstadoTarea());
        if (entity.getRegistrosHoras() != null && entity.getRegistrosHoras().size() > 0) {
            tarea.registrarRegistroHoras(entity.getRegistrosHoras());
            iRepoRegistrosHoras.saveAll(entity.getRegistrosHoras());
            if (tarea.getTipoLabor().isNecesitaProducto()) {
                if (entity.getProductosTareaEmpleados() != null && entity.getProductosTareaEmpleados().size() > 0) {
                    tarea.registarProductoTareaEmpleado(entity.getProductosTareaEmpleados());
                    iRepoProductoTareaEmpleado.saveAll(entity.getProductosTareaEmpleados());
                } else {
                    throw new IllegalArgumentException(Constantes.HAY_QUE_INDICAR_PRODUCTO_EMPLEADO);
                }
            }
            if (tarea.getTipoLabor().isRecolectaProducto()) {
                if (entity.getProductosTareaRecogidos() != null && entity.getProductosTareaRecogidos().size() > 0) {
                    tarea.registarProductoTareaRecogido(entity.getProductosTareaRecogidos());
                    iRepoProductoTareaRecogido.saveAll(entity.getProductosTareaRecogidos());
                } else {
                    throw new IllegalArgumentException(Constantes.HAY_QUE_INDICAR_PRODUCTO_RECOGIDO);
                }
            }
            return tarea;
        } else {
            throw new IllegalArgumentException(Constantes.ALGUNO_DE_LOS_CAMPOS_OBLIGATORIOS_ES_NULO);
        }
    }

    private double diferenciaEntreFechasDias(Tarea tarea) {
        Date hoy = new Date();
        double diffDias = (tarea.getFechaInicio().getTime() - hoy.getTime()) / (24 * 60 * 60 * 1000);
        return diffDias;
    }
}