package com.tfg.gestiontareas.api;

import java.util.Optional;

import com.fasterxml.jackson.annotation.JsonView;
import com.tfg.gestiontareas.model.Usuario;
import com.tfg.gestiontareas.model.UsuarioAgricultor;
import com.tfg.gestiontareas.model.UsuarioCooperativa;
import com.tfg.gestiontareas.repository.IRepoUsuario;
import com.tfg.gestiontareas.util.CorreoDuplicadoException;
import com.tfg.gestiontareas.util.Views;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/public")
public class PublicApiController {

    private Logger logger = LoggerFactory.getLogger(PrivateCooperativaController.class);

    @Autowired
    private IRepoUsuario iRepoUsuario;

    @Autowired
    private BCryptPasswordEncoder passwordEncoder;

    @PostMapping(value = "/agricultores")
    public ResponseEntity<?> registrarAgricultor(@RequestBody UsuarioAgricultor entity) {

        try {
            comprobarUsuario(entity);
            String passwordActual = entity.getPassword();
            entity.setPassword(passwordEncoder.encode(passwordActual));
            UsuarioAgricultor registrado = iRepoUsuario.save(entity);
            return new ResponseEntity<Usuario>(registrado, HttpStatus.CREATED);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(ex.getMessage(), HttpStatus.BAD_REQUEST);
        }
    }

    @JsonView(Views.PasswordView.class)
    @PutMapping(value = "/agricultores/{id}")
    public ResponseEntity<?> modificarPassword(@RequestBody UsuarioAgricultor entity,
            @PathVariable(name = "id") Long id) {
        try {
            Optional<Usuario> optUser = iRepoUsuario.findById(id);
            if (optUser.isPresent()) {
                String nuevaPwd = entity.getPassword();
                Usuario user = optUser.get();
                user.setPassword(passwordEncoder.encode(nuevaPwd));
                Usuario guardado = iRepoUsuario.save(user);
                return new ResponseEntity<Usuario>(guardado, HttpStatus.OK);
            } else {
                return new ResponseEntity<String>("Usuario no encontrado", HttpStatus.NOT_FOUND);
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(ex.getMessage(), HttpStatus.BAD_REQUEST);
        }
    }

    @PostMapping(value = "/cooperativas")
    public ResponseEntity<?> crearCooperativa(@RequestBody UsuarioCooperativa entity) {

        try {
            comprobarUsuario(entity);
            String passwordActual = entity.getPassword();
            entity.setPassword(passwordEncoder.encode(passwordActual));
            UsuarioCooperativa registrado = iRepoUsuario.save(entity);
            return new ResponseEntity<Usuario>(registrado, HttpStatus.CREATED);
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            return new ResponseEntity<String>(ex.getMessage(), HttpStatus.BAD_REQUEST);
        }
    }

    @GetMapping
    public ResponseEntity<?> isUsername(@RequestParam String username, @RequestParam(required = false) Long id) {
        Optional<Usuario> user = iRepoUsuario.findByUsername(username);
        if (user.isPresent()) {
            if (id == null) {
                return new ResponseEntity<Usuario>(user.get(), HttpStatus.OK);
            } else {
                Usuario usuario = user.get();
                return new ResponseEntity<Boolean>(id != usuario.getIdentificador(), HttpStatus.OK);
            }
        } else {
            return new ResponseEntity<Boolean>(false, HttpStatus.OK);
        }
    }

    private void comprobarUsuario(Usuario usuario) throws IllegalArgumentException, CorreoDuplicadoException {

        if (usuario.getNombre() != null || usuario.getUsername() != null || usuario.getPassword() != null) {
            Optional<Usuario> mismoCorreo = iRepoUsuario.findByUsername(usuario.getUsername());
            if (!mismoCorreo.isPresent()) {
                if (usuario instanceof UsuarioAgricultor) {
                    UsuarioAgricultor usuarioAgricultor = (UsuarioAgricultor) usuario;
                    if (usuarioAgricultor.getMunicipio() == null) {
                        throw new IllegalArgumentException(Constantes.ALGUNO_DE_LOS_CAMPOS_OBLIGATORIOS_ES_NULO);
                    }
                } else {
                    UsuarioCooperativa usuarioCooperativa = (UsuarioCooperativa) usuario;
                    if (usuarioCooperativa.getDireccionSede() == null
                            || usuarioCooperativa.getNombreResponsable() == null) {
                        throw new IllegalArgumentException(Constantes.ALGUNO_DE_LOS_CAMPOS_OBLIGATORIOS_ES_NULO);
                    }
                }
            } else {
                throw new CorreoDuplicadoException("Ya existe un usuario con ese correo");
            }
        } else {
            throw new IllegalArgumentException(Constantes.ALGUNO_DE_LOS_CAMPOS_OBLIGATORIOS_ES_NULO);
        }
    }

}