package com.tfg.gestiontareas.model;

import java.util.Set;
import java.util.SortedSet;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonView;
import com.tfg.gestiontareas.util.Views;

@Entity(name = "Parcelas")
public class Parcela {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id", nullable = false, updatable = false)
    @JsonView(Views.ListView.class)
    private Long identificador;

    @Column(name = "nombre", nullable = false)
    @JsonView(Views.ListView.class)
    private String nombre;

    @Column(name = "descripcion", nullable = true)
    private String descripcion;

    @Column(name = "localidad", nullable = false)
    @JsonView(Views.ListView.class)
    private String localidad;

    @Column(name = "superficieHa", nullable = false)
    @JsonView(Views.ListView.class)
    private Float superficieHa;

    @Column(name = "coordLat", nullable = true)
    @JsonView(Views.ListView.class)
    private Float coordLat;

    @Column(name = "coordLong", nullable = true)
    @JsonView(Views.ListView.class)
    private Float coordLong;

    @ManyToOne(optional = false)
    private UsuarioAgricultor propietario;

    @OneToMany(mappedBy = "parcela", orphanRemoval = true, fetch = FetchType.EAGER)
    @JsonIgnore
    @OrderBy("identificador DESC")
    private SortedSet<Tarea> tareas;

    public Parcela() {

    }

    public Long getIdentificador() {
        return identificador;
    }

    public String getNombre() {
        return nombre;
    }

    public void setNombre(String nombre) {
        this.nombre = nombre;
    }

    public String getDescripcion() {
        return descripcion;
    }

    public void setDescripcion(String descripcion) {
        this.descripcion = descripcion;
    }

    public String getLocalidad() {
        return localidad;
    }

    public void setLocalidad(String localidad) {
        this.localidad = localidad;
    }

    public Float getSuperficieHa() {
        return superficieHa;
    }

    public void setSuperficieHa(Float superficieHa) {
        this.superficieHa = superficieHa;
    }

    public Float getCoordLat() {
        return coordLat;
    }

    public void setCoordLat(Float coordLat) {
        this.coordLat = coordLat;
    }

    public Float getCoordLong() {
        return coordLong;
    }

    public void setCoordLong(Float coordLong) {
        this.coordLong = coordLong;
    }

    public UsuarioAgricultor getPropietario() {
        return propietario;
    }

    public void setPropietario(UsuarioAgricultor propietario) {
        this.propietario = propietario;
    }

    public Set<Tarea> getTareas() {
        return tareas;
    }

    public void setTareas(SortedSet<Tarea> tareas) {
        this.tareas = tareas;
    }

    public void registrarTarea(Tarea t) {
        _registrarTarea(t);
    }

    private void _registrarTarea(Tarea tarea) {
        tarea.setParcela(this);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Parcela) {
            if (((Parcela) obj).getIdentificador().equals(identificador)) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

}
