package com.tfg.gestiontareas.model;

import java.util.Date;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;

import com.fasterxml.jackson.annotation.JsonIgnore;

@Entity(name = "ProductosCooperativas")
public class ProductoCooperativa {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id", nullable = false, updatable = false)
    private Long identificador;

    @Column(name = "nombre", nullable = false)
    private String nombre;

    @Column(name = "fechaModificacion", nullable = false)
    private Date fechaModificacion;

    @ManyToOne(optional = false)
    @JsonIgnore
    private UsuarioCooperativa cooperativa;

    @OneToMany(mappedBy = "productoCooperativa", orphanRemoval = true, fetch = FetchType.EAGER)
    private Set<PrecioProducto> preciosProductos;

    public ProductoCooperativa() {
    }

    public Long getIdentificador() {
        return identificador;
    }

    public String getNombre() {
        return nombre;
    }

    public void setNombre(String nombre) {
        this.nombre = nombre;
    }

    public Date getFechaModificacion() {
        return fechaModificacion;
    }

    public void setFechaModificacion(Date fechaModificacion) {
        this.fechaModificacion = fechaModificacion;
    }

    public Set<PrecioProducto> getPreciosProductos() {
        return preciosProductos;
    }

    public void setPreciosProductos(Set<PrecioProducto> preciosProductos) {
        this.preciosProductos = preciosProductos;
    }

    public void setIdentificador(Long identificador) {
        this.identificador = identificador;
    }

    public UsuarioCooperativa getCooperativa() {
        return cooperativa;
    }

    public void setCooperativa(UsuarioCooperativa cooperativa) {
        this.cooperativa = cooperativa;
    }

    public void registrarPrecioProducto(Set<PrecioProducto> precioProducto) {
        precioProducto.forEach(item -> {
            _registrarPrecioProducto(item);
        });
    }

    public void registrarPrecioProducto(PrecioProducto precioProducto) {
        _registrarPrecioProducto(precioProducto);
    }

    private void _registrarPrecioProducto(PrecioProducto precioProducto) {
        precioProducto.setFechaCreacion(new Date());
        precioProducto.setProductoCooperativa(this);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof ProductoCooperativa) {
            if (((ProductoCooperativa) obj).getIdentificador().equals(identificador)) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

}