package com.tfg.gestiontareas.model;

import java.util.Date;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;

import com.fasterxml.jackson.annotation.JsonView;
import com.tfg.gestiontareas.util.Views;

@Entity(name = "Tareas")
public class Tarea implements Comparable<Tarea> {

    public static final float DIAS_PARA_ELIMINAR_TAREA = 2.0f;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id", nullable = false, updatable = false)
    @JsonView(Views.ListView.class)
    private Long identificador;

    @Column(name = "estadoTarea", nullable = false)
    @Enumerated(EnumType.STRING)
    @JsonView(Views.ListView.class)
    private EnumEstadoTarea estadoTarea;

    @Column(name = "fechaInicio", nullable = true)
    private Date fechaInicio;

    @Column(name = "tiempoEstimado", nullable = true)
    private Float tiempoEstimadoHoras;

    @ManyToOne(optional = false)
    @JsonView(Views.ListView.class)
    private TipoLabor tipoLabor;

    @ManyToOne
    private Apero apero;

    @ManyToOne(optional = false)
    private UsuarioAgricultor encargado;

    @ManyToOne(optional = false)
    private Parcela parcela;

    @OneToMany(mappedBy = "tarea", orphanRemoval = true, fetch = FetchType.EAGER)
    private Set<RegistroHoras> registrosHoras;

    @OneToMany(mappedBy = "tarea", orphanRemoval = true, fetch = FetchType.EAGER)
    private Set<ProductoTareaEmpleado> productosTareaEmpleados;

    @OneToMany(mappedBy = "tarea", orphanRemoval = true, fetch = FetchType.EAGER)
    private Set<ProductoTareaRecogido> productosTareaRecogidos;

    public Tarea() {
        setEstadoTarea(EnumEstadoTarea.PTE_ESTIMACION);
    }

    public Long getIdentificador() {
        return identificador;
    }

    public EnumEstadoTarea getEstadoTarea() {
        return estadoTarea;
    }

    public void setEstadoTarea(EnumEstadoTarea estadoTarea) {
        this.estadoTarea = estadoTarea;
    }

    public Date getFechaInicio() {
        return fechaInicio;
    }

    public void setFechaInicio(Date fechaInicio) {
        this.fechaInicio = fechaInicio;
    }

    public Float getTiempoEstimadoHoras() {
        return tiempoEstimadoHoras;
    }

    public void setTiempoEstimadoHoras(Float tiempoEstimadoHoras) {
        this.tiempoEstimadoHoras = tiempoEstimadoHoras;
    }

    public TipoLabor getTipoLabor() {
        return tipoLabor;
    }

    public void setTipoLabor(TipoLabor tipoLabor) {
        this.tipoLabor = tipoLabor;
    }

    public Apero getApero() {
        return apero;
    }

    public void setApero(Apero apero) {
        this.apero = apero;
    }

    public UsuarioAgricultor getEncargado() {
        return encargado;
    }

    public void setEncargado(UsuarioAgricultor encargado) {
        this.encargado = encargado;
    }

    public Parcela getParcela() {
        return parcela;
    }

    public void setParcela(Parcela parcela) {
        this.parcela = parcela;
    }

    public Set<RegistroHoras> getRegistrosHoras() {
        return registrosHoras;
    }

    public void setRegistrosHoras(Set<RegistroHoras> registrosHoras) {
        this.registrosHoras = registrosHoras;
    }

    public Set<ProductoTareaEmpleado> getProductosTareaEmpleados() {
        return productosTareaEmpleados;
    }

    public void setProductosTareaEmpleados(Set<ProductoTareaEmpleado> productosTareaEmpleados) {
        this.productosTareaEmpleados = productosTareaEmpleados;
    }

    public Set<ProductoTareaRecogido> getProductosTareaRecogidos() {
        return productosTareaRecogidos;
    }

    public void setProductosTareaRecogidos(Set<ProductoTareaRecogido> productosTareaRecogidos) {
        this.productosTareaRecogidos = productosTareaRecogidos;
    }

    public void registarProductoTareaEmpleado(ProductoTareaEmpleado productoTareaEmpleado) {
        _registarProductoTareaEmpleado(productoTareaEmpleado);
    }

    public void registarProductoTareaEmpleado(Set<ProductoTareaEmpleado> productoTareaEmpleado) {
        productoTareaEmpleado.forEach(item -> {
            _registarProductoTareaEmpleado(item);
        });
    }

    private void _registarProductoTareaEmpleado(ProductoTareaEmpleado productoTareaEmpleado) {
        productoTareaEmpleado.setTarea(this);
    }

    public void registarProductoTareaRecogido(Set<ProductoTareaRecogido> productosTareaRecogidos) {
        productosTareaRecogidos.forEach(item -> {
            _registarProductoTareaRecogido(item);
        });
    }

    public void registarProductoTareaRecogido(ProductoTareaRecogido productoTareaRecogido) {
        _registarProductoTareaRecogido(productoTareaRecogido);
    }

    private void _registarProductoTareaRecogido(ProductoTareaRecogido productoTareaRecogido) {
        productoTareaRecogido.setTarea(this);
    }

    public void registrarRegistroHoras(Set<RegistroHoras> registroHoras) {
        for (RegistroHoras rh : registroHoras) {
            _registrarRegistroHoras(rh);
        }
    }

    private void _registrarRegistroHoras(RegistroHoras registroHoras) {
        registroHoras.setTarea(this);
    }

    public void modificarEstadoTarea(EnumEstadoTarea estadoTarea) throws IllegalStateException {
        switch (estadoTarea) {
        case PTE_ESTIMACION:
            if (this.estadoTarea != null) {
                throw new IllegalStateException(
                        "Sólo se pueden marcar como pendiente estimación tareas con estado pendiente de estimacion.");
            } else {
                this.estadoTarea = estadoTarea;
            }
            break;
        case ESTIMADA:
            if (!this.estadoTarea.equals(EnumEstadoTarea.PTE_ESTIMACION)) {
                throw new IllegalStateException(
                        "Sólo se pueden marcar como estimadas tareas con estado pendiente de estimacion.");
            } else {
                this.estadoTarea = estadoTarea;
            }
            break;
        case COMPLETADA:
            if (!this.estadoTarea.equals(EnumEstadoTarea.ESTIMADA)) {
                throw new IllegalStateException("Sólo se pueden marcar como completadas tareas con estado estimada.");
            } else {
                this.estadoTarea = estadoTarea;
            }
            break;
        }
    }

    public void modificarAperoTarea(Apero apero) throws IllegalArgumentException {
        boolean mismoTipoLabor = false;
        for (TipoLabor tipoL : apero.getTipoLabores()) {
            if (tipoL.equals(tipoLabor)) {
                mismoTipoLabor = true;
                break;
            }
        }
        if (!mismoTipoLabor) {
            throw new IllegalArgumentException(
                    "Este apero no puede realizar el tipo de labor que requiere esta tarea.");
        } else {
            if (!apero.getPropietario().equals(encargado)) {
                throw new IllegalArgumentException("Este apero no es del encargado de la tarea.");
            } else {
                this.apero = apero;
            }

        }
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Tarea) {
            if (((Tarea) obj).getIdentificador().equals(identificador)) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    @Override
    public int compareTo(Tarea o) {
        return -1 * identificador.compareTo(o.getIdentificador());
    }

}
