package com.tfg.gestiontareas.model;

import java.util.ArrayList;
import java.util.Collection;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.swing.text.View;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.tfg.gestiontareas.util.Views;

import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

@Entity(name = "Usuarios")
@Inheritance(strategy = InheritanceType.JOINED)
@JsonIgnoreProperties(value = { "authorities", "accountNonExpired", "accountNonLocked", "credentialsNonExpired",
        "enabled" })
public class Usuario implements UserDetails {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id", nullable = false, updatable = false)
    @JsonView(Views.ListView.class)
    private Long identificador;

    @Column(name = "username", nullable = false, unique = true)
    @JsonView(Views.ListView.class)
    private String username;

    @Column(name = "password", nullable = false)
    @JsonView(Views.PasswordView.class)
    private String password;

    @Column(name = "nombre", nullable = false)
    @JsonView(Views.ListView.class)
    private String nombre;

    @Column(name = "rol", nullable = false)
    @Enumerated(EnumType.STRING)
    @JsonView(Views.AdminUsersListView.class)
    private EnumRole rol;

    @JsonView(Views.AdminUsersListView.class)
    @Column(name = "isBloqueado", nullable = false)
    private boolean isBloqueado;

    public Usuario() {
        setBloqueado(false);
    }

    public Long getIdentificador() {
        return identificador;
    }

    public String getNombre() {
        return nombre;
    }

    public void setNombre(String nombre) {
        this.nombre = nombre;
    }

    public EnumRole getRol() {
        return rol;
    }

    protected void setRol(EnumRole rol) {
        this.rol = rol;
    }

    public boolean isBloqueado() {
        return isBloqueado;
    }

    public void setBloqueado(boolean isBloqueado) {
        this.isBloqueado = isBloqueado;
    }

    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        Collection<SimpleGrantedAuthority> authorities = new ArrayList<>();
        authorities.add(new SimpleGrantedAuthority(rol.toString()));
        return authorities;
    }

    @Override
    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    @Override
    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    @Override
    public boolean isAccountNonExpired() {
        return true;
    }

    @Override
    public boolean isAccountNonLocked() {
        return !isBloqueado;
    }

    @Override
    public boolean isCredentialsNonExpired() {
        return true;
    }

    @Override
    public boolean isEnabled() {
        return true;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Usuario) {
            if (((Usuario) obj).getIdentificador().equals(identificador)) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    public String generarJWTSubject() {
        return String.format(
                "{\"identificador\":%d, \"username\":\"%s\", \"password\":\"%s\", \"nombre\":\"%s\", \"rol\":\"%s\"}",
                identificador, username, password, nombre, rol);
    }

}