package com.tfg.gestiontareas.model;

import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonView;
import com.tfg.gestiontareas.util.Views;

import javax.persistence.JoinColumn;

@Entity(name = "Agricultores")
public class UsuarioAgricultor extends Usuario {

    private static final long serialVersionUID = -7771612057710344547L;

    @Column(name = "municipio", nullable = false)
    @JsonView(Views.CooperativaUsersListView.class)
    private String municipio;

    @OneToMany(mappedBy = "propietario", orphanRemoval = true, fetch = FetchType.EAGER)
    @JsonIgnore
    private Set<Apero> aperos;

    @OneToMany(mappedBy = "encargado", orphanRemoval = true, fetch = FetchType.EAGER)
    @JsonIgnore
    private Set<Tarea> tareas;

    @OneToMany(mappedBy = "propietario", orphanRemoval = true, fetch = FetchType.EAGER)
    @JsonIgnore
    private Set<Parcela> parcelas;

    @ManyToOne(optional = true)
    private Usuario cooperativa;

    @ManyToMany(fetch = FetchType.EAGER)
    @JoinTable(name = "tbl_agenda", joinColumns = @JoinColumn(name = "agricultorA"), inverseJoinColumns = @JoinColumn(name = "agricultorB"))
    @JsonIgnore
    private Set<UsuarioAgricultor> agenda;

    @ManyToMany(fetch = FetchType.EAGER)
    @JoinTable(name = "tbl_agendaDe", joinColumns = @JoinColumn(name = "agricultorB"), inverseJoinColumns = @JoinColumn(name = "agricultorA"))
    @JsonIgnore
    private Set<UsuarioAgricultor> enAgendaDe;

    public UsuarioAgricultor() {
        super();
        setRol(EnumRole.ROLE_AGRICULTOR);
    }

    public String getMunicipio() {
        return municipio;
    }

    public void setMunicipio(String municipio) {
        this.municipio = municipio;
    }

    public Set<Apero> getAperos() {
        return aperos;
    }

    public void setAperos(Set<Apero> aperos) {
        this.aperos = aperos;
    }

    public Set<Tarea> getTareas() {
        return tareas;
    }

    public void setTareas(Set<Tarea> tareas) {
        this.tareas = tareas;
    }

    public Set<Parcela> getParcelas() {
        return parcelas;
    }

    public void setParcelas(Set<Parcela> parcelas) {
        this.parcelas = parcelas;
    }

    public Usuario getCooperativa() {
        return cooperativa;
    }

    public void setCooperativa(Usuario cooperativa) {
        this.cooperativa = cooperativa;
    }

    public Set<UsuarioAgricultor> getAgenda() {
        return agenda;
    }

    public void setAgenda(Set<UsuarioAgricultor> agenda) {
        this.agenda = agenda;
    }

    public Set<UsuarioAgricultor> getEnAgendaDe() {
        return enAgendaDe;
    }

    public void setEnAgendaDe(Set<UsuarioAgricultor> enAgendaDe) {
        this.enAgendaDe = enAgendaDe;
    }

    public void registrarParcela(Parcela p) {
        _registrarParcela(p);
    }

    private void _registrarParcela(Parcela p) {
        p.setPropietario(this);
    }

    public void registrarApero(Apero a) {
        _registrarApero(a);
    }

    private void _registrarApero(Apero a) {
        a.setPropietario(this);
    }

    @Override
    public boolean equals(Object o) {
        if (o instanceof UsuarioAgricultor) {
            return ((UsuarioAgricultor) o).getIdentificador().equals(getIdentificador());
        } else {
            return false;
        }
    }

}