package com.tfg.gestiontareas.security;

import static com.auth0.jwt.algorithms.Algorithm.HMAC512;

import java.io.IOException;
import java.util.Date;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.auth0.jwt.JWT;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.tfg.gestiontareas.model.Usuario;

import org.springframework.security.authentication.AbstractAuthenticationToken;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

public class AuthenticationFilter extends UsernamePasswordAuthenticationFilter {

    private AuthenticationManager authenticationManager;

    public AuthenticationFilter(AuthenticationManager authenticationManager,
            AuthenticationFailureHandler failureHandler) {
        super();
        this.authenticationManager = authenticationManager;
        setFilterProcessesUrl("/public/login");
        setAuthenticationFailureHandler(failureHandler);
        setPostOnly(true);
    }

    @Override
    public Authentication attemptAuthentication(HttpServletRequest request, HttpServletResponse response)
            throws AuthenticationException {
        ObjectMapper objectMapper = new ObjectMapper();
        try {
            LoginMapper loginMapper = objectMapper.readValue(request.getReader(), LoginMapper.class);

            AbstractAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(
                    loginMapper.getUsername(), loginMapper.getPassword());

            return authenticationManager.authenticate(authenticationToken);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }

    }

    @Override
    protected void successfulAuthentication(HttpServletRequest request, HttpServletResponse response, FilterChain chain,
            Authentication authResult) throws IOException, ServletException {

        Usuario implUserDetails = (Usuario) authResult.getPrincipal();

        byte[] signingKey = Constantes.SECRET.getBytes();

        String token = String.format("{\"token\":\" %s%s\"}", Constantes.TOKEN_PREFIX,
                JWT.create().withSubject(implUserDetails.generarJWTSubject())
                        .withExpiresAt(new Date(System.currentTimeMillis() + Constantes.EXP_TIME))
                        .sign(HMAC512(signingKey)));

        CustomAuthenticationResponse<String> authenticationResponse = new CustomAuthenticationResponse<String>(200,
                token, response);

        authenticationResponse.crearRespuesta();

    }

}
