package com.tfg.gestiontareas.security;

import static com.auth0.jwt.algorithms.Algorithm.HMAC512;

import java.io.IOException;
import java.util.Optional;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.auth0.jwt.JWT;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.tfg.gestiontareas.model.Usuario;
import com.tfg.gestiontareas.repository.IRepoUsuario;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.www.BasicAuthenticationFilter;

public class AuthorizationFilter extends BasicAuthenticationFilter {

    private IRepoUsuario iRepoUsuario;

    private final static Logger LOG = LoggerFactory.getLogger(AuthorizationFilter.class);

    public AuthorizationFilter(AuthenticationManager authenticationManager, IRepoUsuario iRepoUsuario) {
        super(authenticationManager);
        this.iRepoUsuario = iRepoUsuario;
    }

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        UsernamePasswordAuthenticationToken authenticationToken = getAuthentication(request);
        if (authenticationToken == null) {
            chain.doFilter(request, response);
            return;
        }

        SecurityContextHolder.getContext().setAuthentication(authenticationToken);
        chain.doFilter(request, response);
    }

    private UsernamePasswordAuthenticationToken getAuthentication(HttpServletRequest request)
            throws JsonMappingException, JsonProcessingException {

        String token = request.getHeader(Constantes.HEADER_FIELD);
        if ((token != null) && token.startsWith(Constantes.TOKEN_PREFIX)) {
            String tokenSinCabecera = token.replace(Constantes.TOKEN_PREFIX, "");
            String subject = JWT.require(HMAC512(Constantes.SECRET.getBytes())).build().verify(tokenSinCabecera)
                    .getSubject();
            ObjectMapper mapper = new ObjectMapper();
            String userName = mapper.readValue(subject, Usuario.class).getUsername();

            if (userName != null) {
                Optional<Usuario> optionalUsuario = iRepoUsuario.findByUsername(userName);
                if (optionalUsuario.isPresent()) {
                    Usuario principal = optionalUsuario.get();
                    UsernamePasswordAuthenticationToken auth = new UsernamePasswordAuthenticationToken(userName, null,
                            principal.getAuthorities());
                    return auth;
                } else {
                    LOG.error("No se ha encontrado el usuario.");
                    return null;
                }
            } else {
                LOG.error("Login nulo.");
                return null;
            }
        } else {
            LOG.error("No se ha pasado el token.");
            return null;
        }
    }

}