package com.tfg.gestiontareas.security;

import java.util.Arrays;

import com.tfg.gestiontareas.repository.IRepoUsuario;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpMethod;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.CorsConfigurationSource;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;

@Configuration
@EnableWebSecurity
public class SecurityConfig extends WebSecurityConfigurerAdapter {

    @Autowired
    private ImpUserDetailService impUserDetailService;

    @Autowired
    private IRepoUsuario iRepoUsuario;

    @Bean
    public BCryptPasswordEncoder getEncoder() {
        return new BCryptPasswordEncoder(5);
    }

    @Override
    protected void configure(AuthenticationManagerBuilder auth) throws Exception {
        auth.userDetailsService(impUserDetailService).passwordEncoder(getEncoder());
    }

    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http.cors().and().csrf().disable().authorizeRequests().antMatchers("/public/**").permitAll()
                .antMatchers(HttpMethod.PUT, "/private/agricultores/{[0-9]+}").hasAnyRole("AGRICULTOR", "ADMIN")
                .antMatchers("/private/usuarios/**").hasRole("ADMIN").antMatchers("/private/agricultores/**")
                .hasRole("AGRICULTOR").antMatchers(HttpMethod.GET, "/private/tiposLabor/**").hasRole("AGRICULTOR")
                .antMatchers(HttpMethod.POST, "/private/tiposLabor/**").hasRole("ADMIN")
                .antMatchers(HttpMethod.GET, "/private/cooperativas/{[0-9]+}")
                .hasAnyRole("AGRICULTOR", "RESPONSABLE_COOPERATIVA").antMatchers("/private/cooperativas/**")
                .hasRole("RESPONSABLE_COOPERATIVA").antMatchers(HttpMethod.GET, "/private/cooperativas/**/productos/")
                .hasAnyRole("RESPONSABLE_COOPERATIVA", "AGRICULTOR").anyRequest().authenticated().and()
                .addFilter(new AuthenticationFilter(authenticationManager(), new CustomAthenticationFailureHandler()))
                .addFilter(new AuthorizationFilter(authenticationManager(), iRepoUsuario)).sessionManagement()
                .sessionCreationPolicy(SessionCreationPolicy.STATELESS);
    }

    @Bean
    public CorsConfigurationSource corsConfigurationSource() {
        CorsConfiguration configuration = new CorsConfiguration();
        configuration.setAllowedOrigins(Arrays.asList("*"));
        configuration.setAllowedMethods(Arrays.asList("GET", "POST", "PUT", "DELETE"));
        configuration.setAllowedHeaders(Arrays.asList("Authorization", "Content-Type", "content-type",
                "x-requested-with", "Access-Control-Allow-Origin", "Access-Control-Allow-Headers", "x-auth-token",
                "x-app-id", "Origin", "Accept", "X-Requested-With", "Access-Control-Request-Method",
                "Access-Control-Request-Headers"));
        UrlBasedCorsConfigurationSource source = new UrlBasedCorsConfigurationSource();
        source.registerCorsConfiguration("/**", configuration);
        return source;
    }
}
