package com.tfg.gestion_tareas_android.utils;

import android.content.Context;
import android.content.Intent;
import android.content.res.Resources;
import android.net.Uri;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.tfg.gestion_tareas_android.R;
import com.tfg.gestion_tareas_android.models.Tarea;

import java.util.ArrayList;

public class TareaAdapter extends ArrayAdapter<Tarea> {

    private Context context;
    private ArrayList<Tarea> tareas;

    public TareaAdapter(@NonNull Context context, @NonNull ArrayList<Tarea> tareas) {
        super(context, 0, tareas);
        this.context = context;
        this.tareas = tareas;
    }

    @NonNull
    @Override
    public View getView(int position, @Nullable View convertView, @NonNull ViewGroup parent) {
        if(convertView == null)
            convertView = LayoutInflater.from(context).inflate(R.layout.layout_tarea,parent,false);
        final Tarea tarea = tareas.get(position);

        Resources resources = context.getResources();

        TextView parcela = convertView.findViewById(R.id.parcela);
        parcela.setText(String.format("%s %s",resources.getString(R.string.parcela),tarea.getParcela().getNombre()));

        TextView labor = convertView.findViewById(R.id.tipoLabor);
        labor.setText(String.format("%s %s", resources.getString(R.string.labor),tarea.getTipoLabor().getNombre()));

        TextView tiempo = convertView.findViewById(R.id.tiempoEstimado);
        tiempo.setText(String.format("%s %s",resources.getString(R.string.timepoEstimado), fromDecimalToHourMin(tarea.getTiempoEstimadoHoras().toString())));

        ImageView alert = convertView.findViewById(R.id.noUbicacion);
        Button ubicacion = convertView.findViewById(R.id.btnverUbicacion);
        boolean tieneUbicacion = tarea.getParcela().getCoordLat() != null &&  tarea.getParcela().getCoordLong() != null;
        if(tieneUbicacion){
            alert.setVisibility(View.GONE);
            ubicacion.setVisibility(View.VISIBLE);
            ubicacion.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    String lat = String.valueOf(tarea.getParcela().getCoordLat()).replace(',','.');
                    String lon = String.valueOf(tarea.getParcela().getCoordLong()).replace(',','.');
                    String uri = String.format("geo:%s,%s?z=15&q=%s,%s", lat,lon,lat,lon);
                    Uri gmmIntentUri = Uri.parse(uri);
                    Intent mapIntent = new Intent(Intent.ACTION_VIEW, gmmIntentUri);
                    mapIntent.setPackage("com.google.android.apps.maps");
                    context.startActivity(mapIntent);
                }
            });
        }else{
            convertView.setClickable(false);
            alert.setVisibility(View.VISIBLE);
            ubicacion.setVisibility(View.GONE);
        }

        return convertView;
    }

    private String fromDecimalToHourMin(String decimal){
        double aDouble = Double.parseDouble(decimal);
        double horas = (int) (aDouble/1);
        double minutos = (aDouble % 1) * 60;
        return String.format("%d:%d", (long) Math.floor(horas),Math.round(minutos));

    }
}
