package com.tfg.gestion_tareas_android.views;

import androidx.appcompat.app.AppCompatActivity;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.auth0.android.jwt.JWT;
import com.google.gson.JsonIOException;
import com.tfg.gestion_tareas_android.R;
import com.tfg.gestion_tareas_android.api.ApiClient;
import com.tfg.gestion_tareas_android.models.LoginMapper;
import com.tfg.gestion_tareas_android.models.TokenResponse;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;

public class LoginActivity extends AppCompatActivity {

    private EditText username;
    private EditText password;
    private Button entrar;

    Retrofit.Builder builder = new Retrofit.Builder().baseUrl("http://virtual.lab.inf.uva.es:20043").addConverterFactory(GsonConverterFactory.create());
    Retrofit retrofit = builder.build();
    ApiClient apiClient = retrofit.create(ApiClient.class);


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);
        username = (EditText) findViewById(R.id.edit_username);
        password = (EditText) findViewById(R.id.edit_password);
        entrar = (Button) findViewById(R.id.btn_login);

    }


    public void login(View view) {
        ConnectivityManager cm = (ConnectivityManager)this.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetwork = cm.getActiveNetworkInfo();
        boolean estaConectado = activeNetwork != null && activeNetwork.isConnectedOrConnecting();
        if(estaConectado) {
            boolean isCorreoValido = false;
            boolean isPasswordValida = false;
            String usernameText = username.getText().toString();
            String passwordText = password.getText().toString();
            String emailRegex = "^[\\w\\.-]+@([\\w\\-]+\\.)+[A-Z]{2,4}$";
            Pattern pattern = Pattern.compile(emailRegex, Pattern.CASE_INSENSITIVE);
            Matcher matcher = pattern.matcher(usernameText);
            isCorreoValido = matcher.matches();
            isPasswordValida = !TextUtils.isEmpty(passwordText);
            final Resources resources = getResources();
            if (isCorreoValido && isPasswordValida) {
                LoginMapper loginMapper = new LoginMapper(usernameText, passwordText);
                Call<TokenResponse> respuesta = apiClient.login(loginMapper);
                respuesta.enqueue(new Callback<TokenResponse>() {
                    @Override
                    public void onResponse(Call<TokenResponse> call, Response<TokenResponse> response) {
                        int responseStatus = response.code();
                        switch (responseStatus) {
                            case 200:
                                String token = response.body().getToken().replace(" Bearer ", "");
                                JWT jwt = new JWT(token);
                                try {
                                    JSONObject json = new JSONObject(jwt.getSubject());
                                    SharedPreferences preferences = LoginActivity.this.getSharedPreferences("Datos", Context.MODE_PRIVATE);
                                    SharedPreferences.Editor editor = preferences.edit();
                                    editor.putLong("identificadorUser", json.getLong("identificador"));
                                    editor.putString("token", "Bearer " + token);
                                    editor.apply();
                                    Intent intent = new Intent(LoginActivity.this, MainActivity.class);
                                    startActivity(intent);
                                } catch (JSONException e) {
                                    Toast.makeText(getApplicationContext(), "Ha ocurrido un error inesperado.", Toast.LENGTH_LONG).show();
                                } finally {
                                    break;
                                }
                            case 403:
                                Toast.makeText(getApplicationContext(), "Usuario o contraseña erróneos.", Toast.LENGTH_LONG).show();
                                break;
                            default:
                                Log.d("tfg",response.toString());
                                Toast.makeText(getApplicationContext(), "1- Error al conectar con el servidor.", Toast.LENGTH_LONG).show();
                                break;
                        }
                    }

                    @Override
                    public void onFailure(Call<TokenResponse> call, Throwable t) {
                        Log.d("TFG",t.toString());
                        Toast.makeText(getApplicationContext(), "Error al conectar con el servidor.", Toast.LENGTH_LONG).show();
                    }
                });
            } else {
                if (!isCorreoValido) {
                    String error = resources.getString(R.string.username_invalido);
                    username.setError(error);
                }
                if (!isPasswordValida) {
                    String error = resources.getString(R.string.password_invalido);
                    password.setError(error);
                }
            }
        }else{
            Toast.makeText(LoginActivity.this,"No tienes acceso a internet", Toast.LENGTH_LONG).show();
        }
    }


}

