package com.tfg.gestion_tareas_android.views;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import android.Manifest;
import android.annotation.SuppressLint;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.res.Resources;
import android.location.Location;
import android.location.LocationManager;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.gms.tasks.Task;
import com.tfg.gestion_tareas_android.R;
import com.tfg.gestion_tareas_android.api.ApiClient;
import com.tfg.gestion_tareas_android.models.Tarea;
import com.tfg.gestion_tareas_android.utils.OrdenarParcelas;
import com.tfg.gestion_tareas_android.utils.TareaAdapter;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;

@SuppressLint("NewApi")
public class MainActivity extends AppCompatActivity {

    private final int MY_PERMISSIONS_REQUEST_ACCES_FINE_LOCATION = 1;
    private boolean permisoConcedido = false;
    private ListView listaTareas;
    private TextView noHayTareas;
    private ArrayList<Tarea> tareas = new ArrayList<>();

    Retrofit.Builder builder = new Retrofit.Builder().baseUrl("http://virtual.lab.inf.uva.es:20043").addConverterFactory(GsonConverterFactory.create());
    Retrofit retrofit = builder.build();
    ApiClient apiClient = retrofit.create(ApiClient.class);

    FusedLocationProviderClient fusedLocationProviderClient;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        listaTareas = findViewById(R.id.listaTareas);
        noHayTareas = findViewById(R.id.tvNoHayTareas);
        fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(this);
        SharedPreferences sharedPreferences = MainActivity.this.getSharedPreferences("Datos", MODE_PRIVATE);
        Long identificador = sharedPreferences.getLong("identificadorUser", 0);
        String token = sharedPreferences.getString("token", "");
        apiClient.getTareas(identificador, token).enqueue(new Callback<ArrayList<Tarea>>() {
            @Override
            public void onResponse(Call<ArrayList<Tarea>> call, Response<ArrayList<Tarea>> response) {
                tareas = response.body();
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                    if (MainActivity.this.checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED && checkSelfPermission(Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                        if (ContextCompat.checkSelfPermission(MainActivity.this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                            if (ActivityCompat.shouldShowRequestPermissionRationale(MainActivity.this, Manifest.permission.ACCESS_FINE_LOCATION)) {
                                Resources resources = MainActivity.this.getResources();
                                Toast.makeText(MainActivity.this, resources.getText(R.string.permisoUbicacion), Toast.LENGTH_LONG).show();
                                mostrarTareas(false);
                            } else {
                                ActivityCompat.requestPermissions(MainActivity.this, new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, MY_PERMISSIONS_REQUEST_ACCES_FINE_LOCATION);
                            }
                        }
                    } else {
                        mostrarTareas(true);
                    }
                }
            }

            @Override
            public void onFailure(Call<ArrayList<Tarea>> call, Throwable t) {
                Toast.makeText(getApplicationContext(), "No se ha podido recuperar los datos.", Toast.LENGTH_LONG).show();
            }
        });
    }

    private void mostrarTareas(boolean ordenar) {
        if (tareas.size() > 0) {
            if (ordenar) {
                fusedLocationProviderClient.getLastLocation().addOnCompleteListener(this, new OnCompleteListener<Location>() {
                    @Override
                    public void onComplete(@NonNull Task<Location> task) {
                        Location loc = task.getResult();
                        if (loc != null) {
                            tareas.sort(new OrdenarParcelas(loc));
                            TareaAdapter adapter = new TareaAdapter(MainActivity.this, tareas);
                            listaTareas.setDividerHeight(0);
                            listaTareas.setAdapter(adapter);
                        } else {
                            TareaAdapter adapter = new TareaAdapter(MainActivity.this, tareas);
                            listaTareas.setDividerHeight(0);
                            listaTareas.setAdapter(adapter);
                            Toast.makeText(getApplicationContext(), "No se ha podido obtener la ubucación.", Toast.LENGTH_LONG).show();
                        }
                    }
                });
            }else{
                TareaAdapter adapter = new TareaAdapter(MainActivity.this, tareas);
                listaTareas.setDividerHeight(0);
                listaTareas.setAdapter(adapter);
            }
        } else {
            listaTareas.setVisibility(View.GONE);
            noHayTareas.setVisibility(View.VISIBLE);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions,
                                           int[] grantResults) {
        if (requestCode == MY_PERMISSIONS_REQUEST_ACCES_FINE_LOCATION) {
            if (grantResults.length > 0
                    && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                mostrarTareas(true);
            } else {
                mostrarTareas(false);
            }
        }
    }

}
